@@ -19,17 +19,21 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <gst/gst.h>
 
-GstElementDetails gst_autoplugcache_details = {
+GST_DEBUG_CATEGORY_STATIC(debug_category);
+#define GST_CAT_DEFAULT debug_category
+
+GstElementDetails gst_autoplugcache_details = GST_ELEMENT_DETAILS (
   "AutoplugCache",
   "Generic",
-  "LGPL",
   "Data cache for the dynamic autoplugger",
-  VERSION,
-  "Erik Walthinsen <omega@temple-baptist.com>",
-  "(C) 2001 RidgeRun, Inc. (www.ridgerun.com)",
-};
+  "Erik Walthinsen <omega@temple-baptist.com>"
+);
 
 #define GST_TYPE_AUTOPLUGCACHE \
   (gst_autoplugcache_get_type())
@@ -155,6 +159,7 @@ gst_autoplugcache_class_init (GstAutoplugCacheClass *klass)
   gobject_class->get_property = gst_autoplugcache_get_property;
 
   gstelement_class->change_state = gst_autoplugcache_change_state;
+  gst_element_class_set_details (gstelement_class, &gst_autoplugcache_details);
 }
 
 static void
@@ -205,7 +210,7 @@ gst_autoplugcache_loop (GstElement *element)
   /* the first time through, the current_playout pointer is going to be NULL */
   if (cache->current_playout == NULL) {
     /* get a buffer */
-    buf = gst_pad_pull (cache->sinkpad);
+    buf = GST_BUFFER (gst_pad_pull (cache->sinkpad));
     if (GST_IS_EVENT (buf)) {
       gst_pad_event_default (cache->sinkpad, GST_EVENT (buf));
       return;
@@ -222,7 +227,7 @@ gst_autoplugcache_loop (GstElement *element)
     g_signal_emit (G_OBJECT(cache), gst_autoplugcache_signals[FIRST_BUFFER], 0, buf);
 
     /* send the buffer on its way */
-    gst_pad_push (cache->srcpad, buf);
+    gst_pad_push (cache->srcpad, GST_DATA (buf));
   }
   /* the steady state is where the playout is at the front of the cache */
   else if (g_list_previous(cache->current_playout) == NULL) {
@@ -230,19 +235,19 @@ gst_autoplugcache_loop (GstElement *element)
     /* if we've been told to fire an empty signal (after a reset) */
     if (cache->fire_empty) {
       int oldstate = GST_STATE(cache);
-      GST_DEBUG(0,"at front of cache, about to pull, but firing signal");
+      GST_DEBUG ("at front of cache, about to pull, but firing signal");
       gst_object_ref (GST_OBJECT (cache));
       g_signal_emit (G_OBJECT(cache), gst_autoplugcache_signals[CACHE_EMPTY], 0, NULL);
       if (GST_STATE(cache) != oldstate) {
         gst_object_unref (GST_OBJECT (cache));
-        GST_DEBUG(GST_CAT_AUTOPLUG, "state changed during signal, aborting");
+        GST_DEBUG ("state changed during signal, aborting");
 	return;
       }
       gst_object_unref (GST_OBJECT (cache));
     }
 
     /* get a buffer */
-    buf = gst_pad_pull (cache->sinkpad);
+    buf = GST_BUFFER (gst_pad_pull (cache->sinkpad));
     if (GST_IS_EVENT (buf)) {
       gst_pad_event_default (cache->sinkpad, GST_EVENT (buf));
       return;
@@ -257,7 +262,7 @@ gst_autoplugcache_loop (GstElement *element)
     cache->current_playout = cache->cache;
 
     /* send the buffer on its way */
-    gst_pad_push (cache->srcpad, buf);
+    gst_pad_push (cache->srcpad, GST_DATA (buf));
   }
 
   /* otherwise we're trundling through existing cached buffers */
@@ -271,7 +276,7 @@ gst_autoplugcache_loop (GstElement *element)
     }
 
     /* push that buffer */
-    gst_pad_push (cache->srcpad, GST_BUFFER(cache->current_playout->data));
+    gst_pad_push (cache->srcpad, GST_DATA (GST_BUFFER(cache->current_playout->data)));
   }
 }
 
@@ -295,7 +300,7 @@ gst_autoplugcache_set_property (GObject *object, guint prop_id, const GValue *va
   switch (prop_id) {
     case ARG_CAPS_PROXY:
       cache->caps_proxy = g_value_get_boolean (value);
-GST_DEBUG(0,"caps_proxy is %d",cache->caps_proxy);
+GST_DEBUG ("caps_proxy is %d",cache->caps_proxy);
       if (cache->caps_proxy) {
       } else {
       }
@@ -303,7 +308,7 @@ GST_DEBUG(0,"caps_proxy is %d",cache->caps_proxy);
     case ARG_RESET:
       /* no idea why anyone would set this to FALSE, but just in case ;-) */
       if (g_value_get_boolean (value)) {
-        GST_DEBUG(0,"resetting playout pointer");
+        GST_DEBUG ("resetting playout pointer");
         /* reset the playout pointer to the begining again */
         cache->current_playout = cache->cache_start;
         /* now we can fire a signal when the cache runs dry */
@@ -338,23 +343,25 @@ gst_autoplugcache_get_property (GObject *object, guint prop_id, GValue *value, G
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
+  GST_DEBUG_CATEGORY_INIT (debug_category, "AUTOPLUGCACHE", 0, "autoplugcache element");
 
-  factory = gst_element_factory_new ("autoplugcache", GST_TYPE_AUTOPLUGCACHE,
-                                    &gst_autoplugcache_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  if (!gst_element_register (plugin, "autoplugcache", GST_RANK_NONE, GST_TYPE_AUTOPLUGCACHE))
+    return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "autoplugcache",
-  plugin_init
-};
-
+  "an autoplug cache",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
