@@ -40,11 +40,18 @@
  * - fix bugs
  * ...
  */
- 
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include "gstspider.h"
 #include "gstspideridentity.h"
 #include "gstsearchfuncs.h"
- 
+
+GST_DEBUG_CATEGORY (gst_spider_debug);
+#define GST_CAT_DEFAULT gst_spider_debug
+
 /* signals and args */
 enum {
   /* FILL ME */
@@ -103,14 +110,6 @@ static                          GstElementClass	*			parent_class = NULL;
 /* no signals yet
 static guint gst_spider_signals[LAST_SIGNAL] = { 0 };*/
 
-/* let gstreamer know that we have some request pads available */
-GST_PAD_TEMPLATE_FACTORY (gst_spider_src_template_factory,
-  "src%d",
-  GST_PAD_SRC,
-  GST_PAD_REQUEST,
-  NULL
-);  
-
 /* GObject and GStreamer init functions */
 GType
 gst_spider_get_type(void)
@@ -153,7 +152,8 @@ gst_spider_class_init (GstSpiderClass *klass)
   gobject_class->get_property = gst_spider_get_property;
   gobject_class->dispose = gst_spider_dispose;
 
-  gst_element_class_add_pad_template (gstelement_class, gst_spider_src_template_factory());
+  gst_element_class_add_pad_template (gstelement_class, GST_PAD_TEMPLATE_GET (spider_src_factory));
+  gst_element_class_set_details (gstelement_class, &gst_spider_details);
   
   gstelement_class->request_new_pad = GST_DEBUG_FUNCPTR(gst_spider_request_new_pad);
 }
@@ -204,14 +204,13 @@ gst_spider_request_new_pad (GstElement *element, GstPadTemplate *templ, const gc
   
   /* FIXME: use the requested name for the pad */
 
-  gst_object_ref (GST_OBJECT (templ));
-  GST_PAD_PAD_TEMPLATE (returnpad) = templ;
+  gst_object_replace ((GstObject **) &returnpad->padtemplate, (GstObject *) templ);
   
   gst_bin_add (GST_BIN (element), GST_ELEMENT (identity));
   
   returnpad = gst_element_add_ghost_pad (element, returnpad, padname);
   gst_spider_link_new (identity);
-  GST_DEBUG (GST_CAT_ELEMENT_PADS, "successuflly created requested pad %s:%s", GST_DEBUG_PAD_NAME (returnpad));
+  GST_DEBUG ("successuflly created requested pad %s:%s", GST_DEBUG_PAD_NAME (returnpad));
   
   return returnpad;
 }
@@ -278,30 +277,15 @@ gst_spider_unused_elementname (GstBin *bin, const gchar *startwith)
 static void
 gst_spider_link_sometimes (GstElement *src, GstPad *pad, GstSpiderConnection *conn)
 {
-  gboolean restart = FALSE;
   gulong signal_id = conn->signal_id;
-  GstPad *sinkpad = conn->src->sink;
-  GstSpider *spider = GST_SPIDER (GST_OBJECT_PARENT (conn->src));
-  
-  /* check if restarting is necessary */
-  if (gst_element_get_state ((GstElement *) spider) == GST_STATE_PLAYING)
-  {
-    restart = TRUE;
-    gst_element_set_state ((GstElement *) spider, GST_STATE_PAUSED);
-  }
   
   /* try to autoplug the elements */
   if (gst_spider_plug_from_srcpad (conn, pad) != GST_PAD_LINK_REFUSED) {
-    GST_DEBUG (GST_CAT_AUTOPLUG_ATTEMPT, "%s:%s was autoplugged to %s:%s, removing callback", GST_DEBUG_PAD_NAME (pad), GST_DEBUG_PAD_NAME (sinkpad));
+    GST_DEBUG ("%s:%s was autoplugged to %s:%s, removing callback", 
+	       GST_DEBUG_PAD_NAME (pad), GST_DEBUG_PAD_NAME (conn->src->sink));
     g_signal_handler_disconnect (src, signal_id);
     signal_id = 0;
   }
-  
-  /* restart if needed */
-  if (restart)
-  {
-    gst_element_set_state ((GstElement *) spider, GST_STATE_PLAYING);
-  }
 }
 /* create a new link from those two elements */
 static GstSpiderConnection *
@@ -329,7 +313,7 @@ static void
 gst_spider_link_reset (GstSpiderConnection *conn, GstElement *to)
 {
   GstSpider *spider = GST_SPIDER (GST_OBJECT_PARENT (conn->src));
-  GST_DEBUG (GST_CAT_AUTOPLUG, "resetting link from %s to %s, currently at %s to %s", GST_ELEMENT_NAME (spider->sink_ident), 
+  GST_DEBUG ("resetting link from %s to %s, currently at %s to %s", GST_ELEMENT_NAME (spider->sink_ident), 
 	     GST_ELEMENT_NAME (conn->src), GST_ELEMENT_NAME (conn->current), GST_ELEMENT_NAME (to));
   while ((conn->path != NULL) && ((GstElement *) conn->path->data != to))
   {
@@ -447,19 +431,7 @@ gst_spider_identity_plug (GstSpiderIdentity *ident)
         }
 	if ((GstElement *) spider->sink_ident == conn->current)
 	{
-          gboolean restart = FALSE;
-          /* check if restarting is necessary */
-          if (gst_element_get_state ((GstElement *) spider) == GST_STATE_PLAYING)
-          {
-            restart = TRUE;
-            gst_element_set_state ((GstElement *) spider, GST_STATE_PAUSED);
-          }
 	  gst_spider_plug (conn);
-          /* restart if needed */
-          if (restart)
-          {
-            gst_element_set_state ((GstElement *) spider, GST_STATE_PLAYING);
-          }
 	}
       }
     }
@@ -523,11 +495,14 @@ gst_spider_create_and_plug (GstSpiderConnection *conn, GList *plugpath)
       element = (GstElement *) (endelements == NULL ? conn->src : endelements->data);
     } else {
       element = gst_element_factory_create ((GstElementFactory *) plugpath->data, NULL);
+      GST_DEBUG ("Adding element %s of type %s and syncing state with autoplugger", 
+                 GST_ELEMENT_NAME (element), GST_PLUGIN_FEATURE_NAME (plugpath->data));    
       gst_bin_add (GST_BIN (spider), element);
     }
     /* insert and link new element */
-    if (!gst_element_link (conn->current, element))
-    {
+    if (gst_element_link (conn->current, element)) {
+      gst_element_sync_state_with_parent (element);    
+    } else {
       /* check if the src has SOMETIMES templates. If so, link a callback */
       GList *templs = gst_element_get_pad_template_list (conn->current);
 	 
@@ -539,7 +514,7 @@ gst_spider_create_and_plug (GstSpiderConnection *conn, GList *plugpath)
         GstPadTemplate *templ = (GstPadTemplate *) templs->data;
         if ((GST_PAD_TEMPLATE_DIRECTION (templ) == GST_PAD_SRC) && (GST_PAD_TEMPLATE_PRESENCE(templ) == GST_PAD_SOMETIMES))
         {
-          GST_DEBUG (GST_CAT_AUTOPLUG_ATTEMPT, "adding callback to link element %s to %s", GST_ELEMENT_NAME (conn->current), GST_ELEMENT_NAME (conn->src));
+          GST_DEBUG ("adding callback to link element %s to %s", GST_ELEMENT_NAME (conn->current), GST_ELEMENT_NAME (conn->src));
           conn->signal_id = g_signal_connect (G_OBJECT (conn->current), "new_pad", 
 					      G_CALLBACK (gst_spider_link_sometimes), conn);
 	  g_list_free (plugpath);
@@ -547,11 +522,11 @@ gst_spider_create_and_plug (GstSpiderConnection *conn, GList *plugpath)
         }
         templs = g_list_next (templs);
       }
-      GST_DEBUG (GST_CAT_AUTOPLUG_ATTEMPT, "no chance to link element %s to %s", GST_ELEMENT_NAME (conn->current), GST_ELEMENT_NAME (conn->src));
+      GST_DEBUG ("no chance to link element %s to %s", GST_ELEMENT_NAME (conn->current), GST_ELEMENT_NAME (conn->src));
       g_list_free (plugpath);
       return GST_PAD_LINK_REFUSED;
     }
-    GST_DEBUG (GST_CAT_AUTOPLUG_ATTEMPT, "added element %s and attached it to element %s", GST_ELEMENT_NAME (element), GST_ELEMENT_NAME (conn->current));
+    GST_DEBUG ("added element %s and attached it to element %s", GST_ELEMENT_NAME (element), GST_ELEMENT_NAME (conn->current));
     gst_spider_link_add (conn, element);
     if (plugpath != NULL)
       plugpath = g_list_delete_link (plugpath, plugpath);
@@ -580,7 +555,8 @@ gst_spider_find_element_to_plug (GstElement *src, GstElementFactory *fac, GstPad
     {
       /* is the element the pad is linked to of the right type? */
       GstElement *element = GST_PAD_PARENT (pad);
-      if (GST_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element))->elementfactory == fac) {
+      
+      if (G_TYPE_FROM_INSTANCE (element) == gst_element_factory_get_element_type (fac)) {
         return element;
       }
     }
@@ -612,18 +588,11 @@ gst_spider_plug_from_srcpad (GstSpiderConnection *conn, GstPad *srcpad)
   GstElement *startelement = conn->current;
 
   g_assert ((GstElement *) GST_OBJECT_PARENT (srcpad) == conn->current);
-  GST_DEBUG (GST_CAT_AUTOPLUG_ATTEMPT, "trying to plug from %s:%s to %s", 
+  GST_DEBUG ("trying to plug from %s:%s to %s", 
 	     GST_DEBUG_PAD_NAME (srcpad), GST_ELEMENT_NAME (conn->src));
   
   /* find a path from src to sink */
-  /* FIXME: make that if go away and work anyway */
-  if (srcpad == spider->sink_ident->src)
-  {
-    g_assert (GST_RPAD_PEER (spider->sink_ident->sink) != NULL);
-    plugpath = gst_autoplug_sp (gst_pad_get_caps ((GstPad *) GST_RPAD_PEER (spider->sink_ident->sink)), gst_pad_get_caps (conn->src->sink), spider->factories);
-  } else {
-    plugpath = gst_autoplug_sp (gst_pad_get_caps (srcpad), gst_pad_get_caps (conn->src->sink), spider->factories);
-  }
+  plugpath = gst_autoplug_sp (gst_pad_get_caps (srcpad), gst_pad_get_caps (conn->src->sink), spider->factories);
   
   /* prints out the path that was found for plugging */
   /* g_print ("found path from %s to %s:\n", GST_ELEMENT_NAME (conn->current), GST_ELEMENT_NAME (conn->src));
@@ -636,10 +605,10 @@ gst_spider_plug_from_srcpad (GstSpiderConnection *conn, GstPad *srcpad)
     
   /* if there is no way to plug: return */
   if (plugpath == NULL) {
-    GST_DEBUG (GST_CAT_AUTOPLUG_ATTEMPT, "no chance to plug from %s to %s", GST_ELEMENT_NAME (conn->current), GST_ELEMENT_NAME (conn->src));
+    GST_DEBUG ("no chance to plug from %s to %s", GST_ELEMENT_NAME (conn->current), GST_ELEMENT_NAME (conn->src));
     return GST_PAD_LINK_REFUSED;
   }
-  GST_DEBUG (GST_CAT_AUTOPLUG_ATTEMPT, "found a link that needs %d elements", g_list_length (plugpath));
+  GST_DEBUG ("found a link that needs %d elements", g_list_length (plugpath));
 
   /* now remove non-needed elements from the beginning of the path 
    * alter src to point to the new element where we need to start 
@@ -652,7 +621,7 @@ gst_spider_plug_from_srcpad (GstSpiderConnection *conn, GstPad *srcpad)
     plugpath = g_list_delete_link (plugpath, plugpath);
   }
   
-  GST_DEBUG (GST_CAT_AUTOPLUG_ATTEMPT, "%d elements must be inserted to establish the link", g_list_length (plugpath));
+  GST_DEBUG ("%d elements must be inserted to establish the link", g_list_length (plugpath));
   /* create the elements and plug them */
   result = gst_spider_create_and_plug (conn, plugpath);
   
@@ -665,44 +634,35 @@ gst_spider_plug_from_srcpad (GstSpiderConnection *conn, GstPad *srcpad)
   return result;  
 }
 
-GstElementDetails gst_spider_details = {
+GstElementDetails gst_spider_details = GST_ELEMENT_DETAILS (
   "Spider",
   "Generic",
-  "LGPL",
   "Automatically link sinks and sources",
-  VERSION,
-  "Benjamin Otte <in7y118@public.uni-hamburg.de>",
-  "(C) 2002",
-};
+  "Benjamin Otte <in7y118@public.uni-hamburg.de>"
+);
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
+  GST_DEBUG_CATEGORY_INIT (gst_spider_debug, "spider", 0, "spider autoplugging element");
 
-  factory = gst_element_factory_new("spider", GST_TYPE_SPIDER,
-                                   &gst_spider_details);
-  gst_plugin_set_longname (plugin, "Spider autoplugging elements");
-  g_return_val_if_fail(factory != NULL, FALSE);
-
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (spider_src_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  /* add spideridentity */
-  factory = gst_element_factory_new ("spideridentity", GST_TYPE_SPIDER_IDENTITY,
-                                     &gst_spider_identity_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  if (!gst_element_register (plugin, "spider", GST_RANK_SECONDARY, GST_TYPE_SPIDER))
+    return FALSE;
+  if (!gst_element_register (plugin, "spideridentity", GST_RANK_NONE, GST_TYPE_SPIDER_IDENTITY))
+    return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "gstspider",
-  plugin_init
-};
-
-  
-  
+  "a 1:n autoplugger",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
