@@ -27,20 +27,23 @@
 #include <unistd.h>
 #include <stdlib.h>
 
-#include <gstfdsrc.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
+#include "gstfdsrc.h"
 
 #define DEFAULT_BLOCKSIZE	4096
 
-GstElementDetails gst_fdsrc_details = 
-{
+GST_DEBUG_CATEGORY_STATIC (gst_fdsrc_debug);
+#define GST_CAT_DEFAULT gst_fdsrc_debug
+
+GstElementDetails gst_fdsrc_details = GST_ELEMENT_DETAILS (
   "Disk Source",
   "Source/File",
-  "LGPL",
   "Synchronous read from a file",
-  VERSION,
-  "Erik Walthinsen <omega@cse.ogi.edu>",
-  "(C) 1999",
-};
+  "Erik Walthinsen <omega@cse.ogi.edu>"
+);
 
 
 /* FdSrc signals and args */
@@ -55,7 +58,7 @@ enum {
   ARG_BLOCKSIZE,
 };
 
-
+static void		gst_fdsrc_base_init	(gpointer g_class);
 static void		gst_fdsrc_class_init	(GstFdSrcClass *klass);
 static void		gst_fdsrc_init		(GstFdSrc *fdsrc);
 
@@ -64,7 +67,7 @@ static void		gst_fdsrc_set_property	(GObject *object, guint prop_id,
 static void		gst_fdsrc_get_property	(GObject *object, guint prop_id, 
 						 GValue *value, GParamSpec *pspec);
 
-static GstBuffer *	gst_fdsrc_get		(GstPad *pad);
+static GstData *	gst_fdsrc_get		(GstPad *pad);
 
 
 static GstElementClass *parent_class = NULL;
@@ -78,7 +81,7 @@ gst_fdsrc_get_type (void)
   if (!fdsrc_type) {
     static const GTypeInfo fdsrc_info = {
       sizeof(GstFdSrcClass),      
-      NULL,
+      gst_fdsrc_base_init,
       NULL,
       (GClassInitFunc)gst_fdsrc_class_init,
       NULL,
@@ -88,17 +91,26 @@ gst_fdsrc_get_type (void)
       (GInstanceInitFunc)gst_fdsrc_init,
     };
     fdsrc_type = g_type_register_static (GST_TYPE_ELEMENT, "GstFdSrc", &fdsrc_info, 0);
+  
+    GST_DEBUG_CATEGORY_INIT (gst_fdsrc_debug, "fdsrc", 0, "fdsrc element");
   }
   return fdsrc_type;
 }
 
+static void
+gst_fdsrc_base_init (gpointer g_class)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  
+  gst_element_class_set_details (gstelement_class, &gst_fdsrc_details);
+}
 static void
 gst_fdsrc_class_init (GstFdSrcClass *klass) 
 {
   GObjectClass *gobject_class;
 
-  gobject_class = (GObjectClass*)klass;
-
+  gobject_class = G_OBJECT_CLASS (klass);
+  
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FD,
@@ -170,7 +182,7 @@ gst_fdsrc_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
   }
 }
 
-static GstBuffer *
+static GstData *
 gst_fdsrc_get(GstPad *pad)
 {
   GstFdSrc *src;
@@ -188,14 +200,14 @@ gst_fdsrc_get(GstPad *pad)
   /* if nothing was read, we're in eos */
   if (readbytes == 0) {
     gst_element_set_eos (GST_ELEMENT (src));
-    return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+    return GST_DATA (gst_event_new (GST_EVENT_EOS));
   }
 
   if (readbytes == -1) {
-	  g_error ("Error reading from file descriptor. Ending stream.\n");
-	  gst_element_set_eos (GST_ELEMENT (src));
-	  return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
-	  }
+    g_error ("Error reading from file descriptor. Ending stream.\n");
+    gst_element_set_eos (GST_ELEMENT (src));
+    return GST_DATA (gst_event_new (GST_EVENT_EOS));
+  }
   
   GST_BUFFER_OFFSET (buf) = src->curoffset;
   GST_BUFFER_SIZE (buf) = readbytes;
@@ -203,5 +215,5 @@ gst_fdsrc_get(GstPad *pad)
   src->curoffset += readbytes;
 
   /* we're done, return the buffer */
-  return buf;
+  return GST_DATA (buf);
 }
