@@ -27,9 +27,6 @@
 
 #include "gst.h"
 #include "gstqueue.h"
-#ifndef GST_DISABLE_TYPEFIND
-#include "gsttypefind.h"
-#endif /* GST_DISABLE_TYPEFIND */
 #ifndef GST_DISABLE_REGISTRY
 #include "registries/gstxmlregistry.h"
 #endif /* GST_DISABLE_REGISTRY */
@@ -88,11 +85,13 @@ debug_log_handler (const gchar *log_domain,
 enum {
   ARG_VERSION=1,
   ARG_FATAL_WARNINGS,
+#ifndef GST_DISABLE_GST_DEBUG 
   ARG_DEBUG_LEVEL,
   ARG_DEBUG,
   ARG_DEBUG_DISABLE,
   ARG_DEBUG_NO_COLOR,
   ARG_DEBUG_HELP,
+#endif
   ARG_DISABLE_CPU_OPT,
   ARG_PLUGIN_SPEW,
   ARG_PLUGIN_PATH,
@@ -333,16 +332,14 @@ parse_debug_list (const gchar *list)
 static void 
 load_plugin_func (gpointer data, gpointer user_data)
 {
-  gboolean ret;
   GstPlugin *plugin;
   const gchar *filename;
 
   filename = (const gchar *) data;
 
-  plugin = gst_plugin_new (filename);
-  ret = gst_plugin_load_plugin (plugin, NULL);
+  plugin = gst_plugin_load_file (filename, NULL);
 
-  if (ret) {
+  if (plugin) {
     GST_INFO ("Loaded plugin: \"%s\"", filename);
 
     gst_registry_pool_add_plugin (plugin);
@@ -395,9 +392,11 @@ init_pre (void)
   }
   /* we need threading to be enabled right here */
   _gst_debug_init();
-  
+
+#ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, GST_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif /* ENABLE_NLS */
 
 #ifndef GST_DISABLE_REGISTRY
   {
@@ -448,23 +447,13 @@ init_pre (void)
 }
 
 static gboolean
-gst_register_core_elements (GModule *module, GstPlugin *plugin)
+gst_register_core_elements (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
   /* register some standard builtin types */
-  factory = gst_element_factory_new ("bin", gst_bin_get_type (), &gst_bin_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-  factory = gst_element_factory_new ("pipeline", gst_pipeline_get_type (), &gst_pipeline_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-  factory = gst_element_factory_new ("thread", gst_thread_get_type (), &gst_thread_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-  factory = gst_element_factory_new ("queue", gst_queue_get_type (), &gst_queue_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-#ifndef GST_DISABLE_TYPEFIND
-  factory = gst_element_factory_new ("typefind", gst_type_find_get_type (), &gst_type_find_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-#endif /* GST_DISABLE_TYPEFIND */
+  g_assert (gst_element_register (plugin, "bin", GST_RANK_PRIMARY, GST_TYPE_BIN));
+  g_assert (gst_element_register (plugin, "pipeline", GST_RANK_PRIMARY, GST_TYPE_PIPELINE));
+  g_assert (gst_element_register (plugin, "thread", GST_RANK_PRIMARY, GST_TYPE_THREAD));
+  g_assert (gst_element_register (plugin, "queue", GST_RANK_PRIMARY, GST_TYPE_QUEUE));
 
   return TRUE;
 }
@@ -472,8 +461,17 @@ gst_register_core_elements (GModule *module, GstPlugin *plugin)
 static GstPluginDesc plugin_desc = {
   GST_VERSION_MAJOR,           
   GST_VERSION_MINOR,  
-  "gst_core_plugins",
-  gst_register_core_elements
+  "gst_core_elements",
+  "core elements of the GStreamer library",
+  gst_register_core_elements,
+  NULL,
+  VERSION,
+  GST_LICENSE,
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN,
+
+  GST_STRUCT_PADDING_INIT
 };                         
 
 /*
@@ -510,8 +508,8 @@ init_post (void)
   gst_ghost_pad_get_type ();
   gst_element_factory_get_type ();
   gst_element_get_type ();
-  gst_type_factory_get_type ();
   gst_scheduler_factory_get_type ();
+  gst_type_find_factory_get_type ();
   gst_bin_get_type ();
 #ifndef GST_DISABLE_AUTOPLUG
   gst_autoplug_factory_get_type ();
@@ -532,6 +530,8 @@ init_post (void)
   _gst_plugin_register_static (&plugin_desc);
  
   _gst_cpu_initialize (_gst_enable_cpu_opt);
+  _gst_structure_initialize ();
+  _gst_value_initialize ();
   _gst_props_initialize ();
   _gst_caps_initialize ();
   _gst_plugin_initialize ();
@@ -580,20 +580,48 @@ init_post (void)
   return TRUE;
 }
 
+#ifndef GST_DISABLE_GST_DEBUG
+static gint
+sort_by_category_name (gconstpointer a, gconstpointer b)
+{
+  return strcmp (gst_debug_category_get_name ((GstDebugCategory *) a),
+		 gst_debug_category_get_name ((GstDebugCategory *) b));
+}
 static void
 gst_debug_help (void) 
 {
-  GSList *cats, *walk;
+  GSList *list, *walk;
+  GList *list2, *walk2;
+  
+  if (!init_post ())
+    exit (1);
+  
+  walk2 = list2 = gst_registry_pool_plugin_list ();
+  while (walk2) {
+    GstPlugin *plugin = GST_PLUGIN (walk2->data);
+    walk2 = g_list_next (walk2);
+    
+    if (!gst_plugin_is_loaded (plugin)) {
+#ifndef GST_DISABLE_REGISTRY 
+      if (GST_IS_REGISTRY (plugin->manager)) {
+	GST_CAT_LOG (GST_CAT_PLUGIN_LOADING, "loading plugin %s", plugin->desc.name);
+	if (gst_registry_load_plugin (GST_REGISTRY (plugin->manager), plugin) != GST_REGISTRY_OK)
+	  GST_CAT_WARNING (GST_CAT_PLUGIN_LOADING, "loading plugin %s failed", plugin->desc.name);
+      }
+#endif /* GST_DISABLE_REGISTRY */
+    }
+  }
+  g_list_free (list2);
+      
+  list = gst_debug_get_all_categories ();
+  walk = list = g_slist_sort (list, sort_by_category_name);
   
-  walk = cats = gst_debug_get_all_categories ();
-
   g_print ("\n");
   g_print ("name                  level    description\n");
   g_print ("---------------------+--------+--------------------------------\n");
 
   while (walk) {
-    /* unused when debugging is disabled */
-    G_GNUC_UNUSED GstDebugCategory *cat = (GstDebugCategory *) walk->data;
+    GstDebugCategory *cat = (GstDebugCategory *) walk->data;
   
     if (gst_debug_is_colored ()) {
       gchar *color = gst_debug_construct_term_color (cat->color);
@@ -613,9 +641,10 @@ gst_debug_help (void)
     }
     walk = g_slist_next (walk);
   }   
+  g_slist_free (list);
   g_print ("\n");
 }
-  
+#endif  
 
 static void
 init_popt_callback (poptContext context, enum poptCallbackReason reason,
@@ -639,6 +668,7 @@ init_popt_callback (poptContext context, enum poptCallbackReason reason,
       fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
       g_log_set_always_fatal (fatal_mask);
       break;
+#ifndef GST_DISABLE_GST_DEBUG 
     case ARG_DEBUG_LEVEL: {
       gint tmp = 0;
       tmp = strtol (arg, NULL, 0);
@@ -659,6 +689,7 @@ init_popt_callback (poptContext context, enum poptCallbackReason reason,
     case ARG_DEBUG_HELP:
       gst_debug_help ();
       exit (0);
+#endif
     case ARG_DISABLE_CPU_OPT:
       _gst_enable_cpu_opt = FALSE;
       break;
