@@ -20,15 +20,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* #define GST_DEBUG_ENABLED */
-
-#include <gst/gstconfig.h>
 
 #include "gst_private.h"
 
+#include <gst/gstconfig.h>
+
 #include "gstautoplug.h"
-#include "gstregistry.h"
-#include "gstlog.h"
+#include "gstregistrypool.h"
+#include "gstinfo.h"
 
 enum {
   NEW_OBJECT,
@@ -197,7 +196,7 @@ gst_autoplug_to_renderers (GstAutoplug *autoplug, GstCaps *srccaps, GstElement *
  * gst_autoplug_to_renderers_valist:
  * @autoplug: The autoplugger performing the autoplugging.
  * @srccaps: The source capabilities.
- * @target: The target element .
+ * @target: The target element.
  * @va_list: more target elements.
  *
  * Perform the autoplugging procedure on the given autoplugger. 
@@ -327,7 +326,7 @@ gst_autoplug_factory_find (const gchar *name)
 
   g_return_val_if_fail (name != NULL, NULL);
 
-  GST_DEBUG (0,"gstautoplug: find \"%s\"", name);
+  GST_DEBUG ("gstautoplug: find \"%s\"", name);
 
   feature = gst_registry_pool_find_feature (name, GST_TYPE_AUTOPLUG_FACTORY);
   if (feature)
