@@ -29,11 +29,29 @@
 #include <gst/gstobject.h>
 #include <gst/gstpad.h>
 #include <gst/gstclock.h>
+#include <gst/gstplugin.h>
 #include <gst/gstpluginfeature.h>
 #include <gst/gstindex.h>
 
 G_BEGIN_DECLS
 
+typedef struct _GstElementDetails GstElementDetails;
+
+/* FIXME: need translatable stuff in here (how handle in registry)? */
+struct _GstElementDetails {
+  gchar *longname;              /* long, english name */
+  gchar *klass;                 /* type of element, as hierarchy */
+  gchar *description;           /* insights of one form or another */
+  gchar *author;                /* who wrote this thing? */
+
+  GST_STRUCT_PADDING
+};
+#define GST_ELEMENT_DETAILS(longname,klass,description,author)		\
+  { longname, klass, description, author, GST_STRUCT_PADDING_INIT }
+#define GST_IS_ELEMENT_DETAILS(details) (					\
+  (details) && ((details)->longname != NULL) && ((details)->klass != NULL)	\
+  && ((details)->description != NULL) && ((details)->author != NULL))
+
 #define GST_NUM_STATES 4
 
 /* NOTE: this probably should be done with an #ifdef to decide 
@@ -108,6 +126,12 @@ typedef enum {
   GST_ELEMENT_SCHEDULER_PRIVATE1,
   GST_ELEMENT_SCHEDULER_PRIVATE2,
 
+  /* ignore state changes from parent */
+  GST_ELEMENT_LOCKED_STATE,
+
+  /* element is in error */
+  GST_ELEMENT_ERROR,
+
   /* use some padding for future expansion */
   GST_ELEMENT_FLAG_LAST		= GST_OBJECT_FLAG_LAST + 16
 } GstElementFlags;
@@ -160,14 +184,18 @@ struct _GstElement {
   GAsyncQueue		*prop_value_queue;
   GMutex		*property_mutex;
 
-  gpointer		dummy[8];
+  GST_OBJECT_PADDING
 };
 
 struct _GstElementClass {
   GstObjectClass 	parent_class;
 
-  /* the elementfactory that created us */
-  GstElementFactory 	*elementfactory;
+  /* the element details */
+  GstElementDetails 	details;
+
+  /* factory that the element was created from */
+  GstElementFactory	*elementfactory;
+
   /* templates for our pads */
   GList 		*padtemplates;
   gint 			numpadtemplates;
@@ -212,20 +240,20 @@ struct _GstElementClass {
   GstIndex*		(*get_index)		(GstElement *element);
   void			(*set_index)		(GstElement *element, GstIndex *index);
 
-  /* padding */
-  gpointer		dummy[8];
+  GST_CLASS_PADDING
 };
 
 void			gst_element_class_add_pad_template	(GstElementClass *klass, GstPadTemplate *templ);
 void                    gst_element_class_install_std_props	(GstElementClass *klass,
 								 const gchar      *first_name, ...);
+void			gst_element_class_set_details		(GstElementClass *klass,
+								 GstElementDetails *details);
 
 #define 		gst_element_default_deep_notify 	gst_object_default_deep_notify
 
 void 			gst_element_default_error		(GObject *object, GstObject *orig, gchar *error);
 
 GType			gst_element_get_type		(void);
-#define			gst_element_destroy(element)	gst_object_destroy (GST_OBJECT (element))
 
 void			gst_element_set_loop_function	(GstElement *element,
 							 GstElementLoopFunction loop);
@@ -258,6 +286,7 @@ GstClock*		gst_element_get_clock 		(GstElement *element);
 void			gst_element_set_clock 		(GstElement *element, GstClock *clock);
 GstClockReturn		gst_element_clock_wait 		(GstElement *element, 
 							 GstClockID id, GstClockTimeDiff *jitter);
+ 
 /* indexs */
 gboolean		gst_element_is_indexable	(GstElement *element);
 void			gst_element_set_index		(GstElement *element, GstIndex *index);
@@ -286,27 +315,33 @@ GstPad*			gst_element_get_compatible_pad	(GstElement *element, GstPad *pad);
 GstPad*			gst_element_get_compatible_pad_filtered (GstElement *element, GstPad *pad, 
 							 GstCaps *filtercaps);
 
+GstPadTemplate*		gst_element_class_get_pad_template	(GstElementClass *element_class, const gchar *name);
+GList*                  gst_element_class_get_pad_template_list (GstElementClass *element_class);
 GstPadTemplate*		gst_element_get_pad_template		(GstElement *element, const gchar *name);
 GList*			gst_element_get_pad_template_list	(GstElement *element);
 GstPadTemplate*		gst_element_get_compatible_pad_template (GstElement *element, GstPadTemplate *compattempl);
 
 gboolean		gst_element_link		(GstElement *src, GstElement *dest);
-gboolean		gst_element_link_many 	(GstElement *element_1, GstElement *element_2, ...);
+gboolean		gst_element_link_many 		(GstElement *element_1, 
+							 GstElement *element_2, ...);
 gboolean		gst_element_link_filtered 	(GstElement *src, GstElement *dest,
 							 GstCaps *filtercaps);
 void			gst_element_unlink 		(GstElement *src, GstElement *dest);
-void			gst_element_unlink_many 	(GstElement *element_1, GstElement *element_2, ...);
+void			gst_element_unlink_many 	(GstElement *element_1, 
+							 GstElement *element_2, ...);
 
-gboolean		gst_element_link_pads	(GstElement *src, const gchar *srcpadname,
+gboolean		gst_element_link_pads		(GstElement *src, const gchar *srcpadname,
 							 GstElement *dest, const gchar *destpadname);
-gboolean		gst_element_link_pads_filtered (GstElement *src, const gchar *srcpadname,
+gboolean		gst_element_link_pads_filtered 	(GstElement *src, const gchar *srcpadname,
 							 GstElement *dest, const gchar *destpadname,
 							 GstCaps *filtercaps);
-void			gst_element_unlink_pads	(GstElement *src, const gchar *srcpadname,
+void			gst_element_unlink_pads		(GstElement *src, const gchar *srcpadname,
 							 GstElement *dest, const gchar *destpadname);
 
 const GstEventMask*	gst_element_get_event_masks	(GstElement *element);
 gboolean		gst_element_send_event		(GstElement *element, GstEvent *event);
+gboolean		gst_element_seek		(GstElement *element, GstSeekType seek_type,
+							 guint64 offset);
 const GstQueryType*	gst_element_get_query_types	(GstElement *element);
 gboolean		gst_element_query		(GstElement *element, GstQueryType type,
 			                                 GstFormat *format, gint64 *value);
@@ -319,6 +354,10 @@ void			gst_element_set_eos		(GstElement *element);
 
 void 			gst_element_error 		(GstElement *element, const gchar *error, ...);
 
+gboolean		gst_element_is_locked_state	(GstElement *element);
+void			gst_element_set_locked_state	(GstElement *element, gboolean locked_state);
+gboolean		gst_element_sync_state_with_parent (GstElement *element);
+
 GstElementState         gst_element_get_state           (GstElement *element);
 GstElementStateReturn	gst_element_set_state		(GstElement *element, GstElementState state);
 
@@ -336,17 +375,6 @@ GstBin*			gst_element_get_managing_bin	(GstElement *element);
  * factories stuff
  *
  **/
-typedef struct _GstElementDetails GstElementDetails;
-
-struct _GstElementDetails {
-  gchar *longname;              /* long, english name */
-  gchar *klass;                 /* type of element, as hierarchy */
-  gchar *license;               /* license element is under */
-  gchar *description;           /* insights of one form or another */
-  gchar *version;               /* version of the element */
-  gchar *author;                /* who wrote this thing? */
-  gchar *copyright;             /* copyright details (year, etc.) */
-};
 
 #define GST_TYPE_ELEMENT_FACTORY 		(gst_element_factory_get_type())
 #define GST_ELEMENT_FACTORY(obj)  		(G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_ELEMENT_FACTORY,\
@@ -356,50 +384,54 @@ struct _GstElementDetails {
 #define GST_IS_ELEMENT_FACTORY(obj) 		(G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ELEMENT_FACTORY))
 #define GST_IS_ELEMENT_FACTORY_CLASS(klass) 	(G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ELEMENT_FACTORY))
 
-#define GST_ELEMENT_RANK_PRIMARY    256
-#define GST_ELEMENT_RANK_SECONDARY  128
-#define GST_ELEMENT_RANK_MARGINAL   64
-#define GST_ELEMENT_RANK_NONE       0
-
 struct _GstElementFactory {
-  GstPluginFeature feature;
+  GstPluginFeature	parent;
 
-  GType type;			/* unique GType of element */
+  GType			type;			/* unique GType of element or 0 if not loaded */
 
-  guint details_dynamic : 1;
+  GstElementDetails	details;
 
-  GstElementDetails *details;	/* pointer to details struct */
+  GList *		padtemplates;
+  guint			numpadtemplates;
 
-  GList *padtemplates;
-  guint16 numpadtemplates;
-
-  guint16 rank;			/* used by autoplug to prioritise elements to try */
+  GST_OBJECT_PADDING
 };
 
 struct _GstElementFactoryClass {
   GstPluginFeatureClass parent_class;
+
+  GST_CLASS_PADDING
 };
 
 GType 			gst_element_factory_get_type 		(void);
 
-GstElementFactory*	gst_element_factory_new			(const gchar *name, GType type,
-                                                                 GstElementDetails *details);
-GstElementFactory*	gst_element_factory_find		(const gchar *name);
+gboolean		gst_element_register			(GstPlugin *plugin,
+								 const gchar *elementname,
+								 guint rank,
+								 GType type);
 
-void			gst_element_factory_add_pad_template	(GstElementFactory *elementfactory,
-								 GstPadTemplate *templ);
+GstElementFactory *	gst_element_factory_find		(const gchar *name);
+GType			gst_element_factory_get_element_type	(GstElementFactory *factory);
+G_CONST_RETURN gchar *	gst_element_factory_get_longname	(GstElementFactory *factory);
+G_CONST_RETURN gchar *	gst_element_factory_get_klass		(GstElementFactory *factory);
+G_CONST_RETURN gchar *	gst_element_factory_get_description  	(GstElementFactory *factory);
+G_CONST_RETURN gchar *	gst_element_factory_get_version      	(GstElementFactory *factory);
+G_CONST_RETURN gchar *	gst_element_factory_get_author		(GstElementFactory *factory);
+guint			gst_element_factory_get_num_pad_templates (GstElementFactory *factory);
+G_CONST_RETURN GList *	gst_element_factory_get_pad_templates	(GstElementFactory *factory);
+
+GstElement*		gst_element_factory_create		(GstElementFactory *factory,
+								 const gchar *name);
+GstElement*		gst_element_factory_make		(const gchar *factoryname, const gchar *name);
 
 gboolean		gst_element_factory_can_src_caps	(GstElementFactory *factory,
 								 GstCaps *caps);
 gboolean		gst_element_factory_can_sink_caps	(GstElementFactory *factory,
 								 GstCaps *caps);
 
-GstElement*		gst_element_factory_create		(GstElementFactory *factory,
-								 const gchar *name);
-GstElement*		gst_element_factory_make		(const gchar *factoryname, const gchar *name);
-GstElement*		gst_element_factory_make_or_warn	(const gchar *factoryname, const gchar *name);
+void			__gst_element_factory_add_pad_template	(GstElementFactory *elementfactory,
+								 GstPadTemplate *templ);
 
-void			gst_element_factory_set_rank		(GstElementFactory *factory, guint16 rank);
 
 G_END_DECLS
 
