@@ -1,6 +1,7 @@
 /* GStreamer
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2000 Wim Taymans <wtay@chello.be>
+ *                    2003 Benjamin Otte <in7y118@public.uni-hamburg.de>
  *
  * gstelementfactory.c: GstElementFactory object, support routines
  *
@@ -20,12 +21,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* #define DEBUG_ENABLED */
 #include "gst_private.h"
 
 #include "gstelement.h"
-#include "gstregistry.h"
-#include "gstlog.h"
+#include "gstregistrypool.h"
+#include "gstinfo.h"
+
+GST_DEBUG_CATEGORY_STATIC (element_factory_debug);
+#define GST_CAT_DEFAULT element_factory_debug
 
 static void 		gst_element_factory_class_init 		(GstElementFactoryClass *klass);
 static void 		gst_element_factory_init 		(GstElementFactory *factory);
@@ -55,10 +58,12 @@ gst_element_factory_get_type (void)
     };
     elementfactory_type = g_type_register_static (GST_TYPE_PLUGIN_FEATURE, 
 		    				  "GstElementFactory", &elementfactory_info, 0);
+    GST_DEBUG_CATEGORY_INIT (element_factory_debug, "GST_ELEMENT_FACTORY", 
+	    GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED,
+	    "element factories keep information about installed elements");
   }
   return elementfactory_type;
 }
-
 static void
 gst_element_factory_class_init (GstElementFactoryClass *klass)
 {
@@ -70,19 +75,16 @@ gst_element_factory_class_init (GstElementFactoryClass *klass)
   gstobject_class = (GstObjectClass*)klass;
   gstpluginfeature_class = (GstPluginFeatureClass*) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_PLUGIN_FEATURE);
+  parent_class = g_type_class_peek_parent (klass);
 
   gstpluginfeature_class->unload_thyself = 	GST_DEBUG_FUNCPTR (gst_element_factory_unload_thyself);
-
 }
-
 static void
 gst_element_factory_init (GstElementFactory *factory)
 {
   factory->padtemplates = NULL;
   factory->numpadtemplates = 0;
 }
-
 /**
  * gst_element_factory_find:
  * @name: name of factory to find
@@ -103,94 +105,97 @@ gst_element_factory_find (const gchar *name)
     return GST_ELEMENT_FACTORY (feature);
 
   /* this should be an ERROR */
-  GST_DEBUG (GST_CAT_ELEMENT_FACTORY,"no such elementfactory \"%s\"", name);
+  GST_DEBUG ("no such elementfactory \"%s\"", name);
   return NULL;
 }
 
-static void
-gst_element_details_free (GstElementDetails *dp)
+void
+__gst_element_details_clear (GstElementDetails *dp)
 {
   g_free (dp->longname);
+  dp->longname = NULL;
   g_free (dp->klass);
-  g_free (dp->license);
+  dp->klass = NULL;
   g_free (dp->description);
-  g_free (dp->version);
+  dp->description = NULL;
   g_free (dp->author);
-  g_free (dp->copyright);
-  g_free (dp);
+  dp->author = NULL;
+}
+void
+__gst_element_details_set (GstElementDetails *dest, const GstElementDetails *src)
+{
+  dest->longname = g_strdup (src->longname);
+  dest->klass = g_strdup (src->klass);
+  dest->description = g_strdup (src->description);
+  dest->author = g_strdup (src->author);
+}
+void
+__gst_element_details_copy (GstElementDetails *dest, const GstElementDetails *src)
+{
+  __gst_element_details_clear (dest);
+  __gst_element_details_set (dest, src);
 }
-
 static void
 gst_element_factory_cleanup (GstElementFactory *factory)
 {
-  GList *padtemplates;
-
-  if (factory->details_dynamic) {
-    gst_element_details_free (factory->details);
-    factory->details_dynamic = FALSE;
+  __gst_element_details_clear (&factory->details);
+  if (factory->type) {
+    g_type_class_unref (g_type_class_peek (factory->type));
+    factory->type = 0;
   }
 
-  padtemplates = factory->padtemplates;
-
-  while (padtemplates) {
-    GstPadTemplate *oldtempl = GST_PAD_TEMPLATE (padtemplates->data);
-
-    /* We leak the padtemplates, since they're generally not properly
-     * ref'ed by the factories */
-    /* gst_object_unref (GST_OBJECT (oldtempl)); */
-
-    padtemplates = g_list_next (padtemplates);
-  }
+  g_list_foreach (factory->padtemplates, (GFunc) g_object_unref, NULL);
   g_list_free (factory->padtemplates);
-
   factory->padtemplates = NULL;
   factory->numpadtemplates = 0;
-
-  g_free (GST_PLUGIN_FEATURE (factory)->name);
 }
-
 /**
- * gst_element_factory_new:
- * @name: name of new elementfactory
- * @type: GType of new element
- * @details: #GstElementDetails structure with element details
+ * gst_element_register:
+ * @plugin:
+ * @name: name of elements of this type
+ * @rank: rank of element (higher rank means more importance when autoplugging)
+ * @type: GType of element to register
  *
  * Create a new elementfactory capable of insantiating objects of the
  * given type.
  *
- * Returns: new elementfactory
+ * Returns: TRUE, if the registering succeeded, FALSE on error
  */
-GstElementFactory*
-gst_element_factory_new (const gchar *name, GType type,
-                        GstElementDetails *details)
+gboolean
+gst_element_register (GstPlugin *plugin, const gchar *name, guint rank, GType type)
 {
   GstElementFactory *factory;
+  GstElementClass *klass;
 
-  g_return_val_if_fail (name != NULL, NULL);
-  g_return_val_if_fail (type, NULL);
-  g_return_val_if_fail (details, NULL);
+  g_return_val_if_fail (name != NULL, FALSE);
+  g_return_val_if_fail (g_type_is_a (type, GST_TYPE_ELEMENT), FALSE);
 
   factory = gst_element_factory_find (name);
 
-  if (!factory)
+  if (!factory) {
+    klass = GST_ELEMENT_CLASS (g_type_class_ref (type));
     factory = GST_ELEMENT_FACTORY (g_object_new (GST_TYPE_ELEMENT_FACTORY, NULL));
-  else {
+    gst_plugin_feature_set_name (GST_PLUGIN_FEATURE (factory), name);
+    GST_LOG_OBJECT (factory, "Created new elementfactory for type %s", g_type_name (type));
+  } else {
+    g_return_val_if_fail (factory->type == 0, FALSE);
+    klass = GST_ELEMENT_CLASS (g_type_class_ref (type));
     gst_element_factory_cleanup (factory);
+    GST_LOG_OBJECT (factory, "Reuse existing elementfactory for type %s", g_type_name (type));
   }
 
-  factory->details = details;
-  factory->details_dynamic = FALSE;
+  factory->type = type;
+  __gst_element_details_copy (&factory->details, &klass->details);
+  factory->padtemplates = g_list_copy (klass->padtemplates);
+  g_list_foreach (factory->padtemplates, (GFunc) g_object_ref, NULL);
+  factory->numpadtemplates = klass->numpadtemplates;
 
-  if (!factory->type)
-    factory->type = type;
-  else if (factory->type != type)
-    g_critical ("`%s' requested type change (!)", name);
+  gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
 
-  GST_PLUGIN_FEATURE (factory)->name = g_strdup (name);
+  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
 
-  return factory;
+  return TRUE;
 }
-
 /**
  * gst_element_factory_create:
  * @factory: factory to instantiate
@@ -200,7 +205,7 @@ gst_element_factory_new (const gchar *name, GType type,
  * It will be given the name supplied, since all elements require a name as
  * their first argument.
  *
- * Returns: new #GstElement
+ * Returns: new #GstElement or NULL if the element couldn't be created
  */
 GstElement*
 gst_element_factory_create (GstElementFactory *factory,
@@ -211,12 +216,13 @@ gst_element_factory_create (GstElementFactory *factory,
 
   g_return_val_if_fail (factory != NULL, NULL);
 
-  if (!gst_plugin_feature_ensure_loaded (GST_PLUGIN_FEATURE (factory)))
+  if (!gst_plugin_feature_ensure_loaded (GST_PLUGIN_FEATURE (factory))) {
+    GST_INFO ("could not load element factory for element \"%s\"", name);
     return NULL;
+  }
 
-  GST_DEBUG (GST_CAT_ELEMENT_FACTORY,
-             "creating element from factory \"%s\" (name \"%s\", type %d)", 
-             GST_PLUGIN_FEATURE_NAME (factory), name, (gint) factory->type);
+  GST_LOG_OBJECT (factory, "creating element (name \"%s\", type %d)", 
+           GST_STR_NULL (name), (gint) factory->type);
 
   if (factory->type == 0) {
       g_critical ("Factory for `%s' has no type",
@@ -224,28 +230,22 @@ gst_element_factory_create (GstElementFactory *factory,
       return NULL;
   }
 
+  oclass = GST_ELEMENT_CLASS (g_type_class_ref (factory->type)); 	 
+  if (oclass->elementfactory == NULL) { 	 
+    GST_DEBUG ("class %s", GST_PLUGIN_FEATURE_NAME (factory)); 	 
+    oclass->elementfactory = factory;
+  }
+
   /* create an instance of the element */
   element = GST_ELEMENT (g_object_new (factory->type, NULL));
   g_assert (element != NULL);
 
-  /* attempt to set the elemenfactory class pointer if necessary */
-  oclass = GST_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element));
-  if (oclass->elementfactory == NULL) {
-    GST_DEBUG (GST_CAT_ELEMENT_FACTORY, "class %s", GST_PLUGIN_FEATURE_NAME (factory));
-    oclass->elementfactory = factory;
-
-    /* copy pad template pointers to the element class, 
-     * allow for custom padtemplates */
-    oclass->padtemplates = g_list_concat (oclass->padtemplates, 
-		    g_list_copy (factory->padtemplates));
-    oclass->numpadtemplates += factory->numpadtemplates;
-  }
+  g_type_class_unref (oclass);
 
   gst_object_set_name (GST_OBJECT (element), name);
 
   return element;
 }
-
 /**
  * gst_element_factory_make:
  * @factoryname: a named factory to instantiate
@@ -256,7 +256,7 @@ gst_element_factory_create (GstElementFactory *factory,
  * consisting of the element factory name and a number.
  * If name is given, it will be given the name supplied.
  *
- * Returns: new #GstElement (or NULL if unable to create element)
+ * Returns: new #GstElement or NULL if unable to create element
  */
 GstElement*
 gst_element_factory_make (const gchar *factoryname, const gchar *name)
@@ -266,61 +266,27 @@ gst_element_factory_make (const gchar *factoryname, const gchar *name)
 
   g_return_val_if_fail (factoryname != NULL, NULL);
 
-  GST_DEBUG (GST_CAT_ELEMENT_FACTORY, "gstelementfactory: make \"%s\" \"%s\"", 
-             factoryname, name);
+  GST_LOG ("gstelementfactory: make \"%s\" \"%s\"", 
+           factoryname, GST_STR_NULL (name));
 
   /* gst_plugin_load_element_factory (factoryname); */
   factory = gst_element_factory_find (factoryname);
   if (factory == NULL) {
-    GST_INFO (GST_CAT_ELEMENT_FACTORY,"no such element factory \"%s\"!",
+    GST_INFO ("no such element factory \"%s\"!",
 	      factoryname);
     return NULL;
   }
   element = gst_element_factory_create (factory, name);
   if (element == NULL) {
-    GST_INFO (GST_CAT_ELEMENT_FACTORY,
-	      "couldn't create instance of element factory \"%s\"!",
-	      factoryname);
+    GST_INFO_OBJECT (factory, "couldn't create instance!");
     return NULL;
   }
 
   return element;
 }
-
-/**
- * gst_element_factory_make_or_warn:
- * @factoryname: a named factory to instantiate
- * @name: name of new element
- *
- * Create a new element of the type defined by the given element factory
- * using #gst_element_factory_make.
- * Will use g_warning if the element could not be created.
- *
- * Returns: new #GstElement (or NULL if unable to create element)
- */
-GstElement*
-gst_element_factory_make_or_warn (const gchar *factoryname, const gchar *name)
-{
-  GstElement *element;
-  
-  element = gst_element_factory_make (factoryname, name);
-
-  if (element == NULL) 
-    g_warning ("Could not create element from factory %s !\n", factoryname);
-
-  return element;
-}
-    
-/**
- * gst_element_factory_add_pad_template :
- * @elementfactory: factory to add the src id to
- * @templ: the padtemplate to add
- *
- * Add the given padtemplate to this elementfactory.
- */
 void
-gst_element_factory_add_pad_template (GstElementFactory *factory,
-			            GstPadTemplate *templ)
+__gst_element_factory_add_pad_template (GstElementFactory *factory,
+					GstPadTemplate *templ)
 {
   g_return_if_fail (factory != NULL);
   g_return_if_fail (templ != NULL);
@@ -331,7 +297,111 @@ gst_element_factory_add_pad_template (GstElementFactory *factory,
   factory->padtemplates = g_list_append (factory->padtemplates, templ);
   factory->numpadtemplates++;
 }
+/**
+ * gst_element_factory_get_element_type:
+ * @factory: factory to get managed #GType from
+ * 
+ * Get the #GType for elements managed by this factory
+ *
+ * Returns: the #GType for elements managed by this factory
+ */
+GType
+gst_element_factory_get_element_type (GstElementFactory *factory)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), 0);
 
+  return factory->type;
+}
+/**
+ * gst_element_factory_get_longname:
+ * @factory: a #GstElementFactory
+ * 
+ * Gets the longname for this factory
+ *
+ * Returns: the longname
+ */
+G_CONST_RETURN gchar *
+gst_element_factory_get_longname (GstElementFactory *factory)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), NULL);
+
+  return factory->details.longname;
+}
+/**
+ * gst_element_factory_get_class:
+ * @factory: a #GstElementFactory
+ * 
+ * Gets the class for this factory.
+ *
+ * Returns: the class
+ */
+G_CONST_RETURN gchar *
+gst_element_factory_get_klass (GstElementFactory *factory)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), NULL);
+
+  return factory->details.klass;
+}
+/**
+ * gst_element_factory_get_description:
+ * @factory: a #GstElementFactory
+ * 
+ * Gets the description for this factory.
+ *
+ * Returns: the description
+ */
+G_CONST_RETURN gchar *
+gst_element_factory_get_description (GstElementFactory *factory)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), NULL);
+
+  return factory->details.description;
+}
+/**
+ * gst_element_factory_get_author:
+ * @factory: a #GstElementFactory
+ * 
+ * Gets the author for this factory.
+ *
+ * Returns: the author
+ */
+G_CONST_RETURN gchar *
+gst_element_factory_get_author (GstElementFactory *factory)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), NULL);
+
+  return factory->details.author;
+}
+/**
+ * gst_element_factory_get_num_pad_templates:
+ * @factory: a #GstElementFactory
+ * 
+ * Gets the number of pad_templates in this factory.
+ *
+ * Returns: the number of pad_templates
+ */
+guint
+gst_element_factory_get_num_pad_templates (GstElementFactory *factory)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), 0);
+
+  return factory->numpadtemplates;
+}
+/**
+ * gst_element_factory_get_pad_templates:
+ * @factory: a #GstElementFactory
+ * 
+ * Gets the #Glist of pad templates for this factory.
+ *
+ * Returns: the padtemplates
+ */
+G_CONST_RETURN GList *
+gst_element_factory_get_pad_templates (GstElementFactory *factory)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), NULL);
+
+  return factory->padtemplates;
+}
 /**
  * gst_element_factory_can_src_caps :
  * @factory: factory to query
@@ -364,7 +434,6 @@ gst_element_factory_can_src_caps (GstElementFactory *factory,
 
   return FALSE;
 }
-
 /**
  * gst_element_factory_can_sink_caps :
  * @factory: factory to query
@@ -397,23 +466,6 @@ gst_element_factory_can_sink_caps (GstElementFactory *factory,
 
   return FALSE;
 }
-
-/**
- * gst_element_factory_set_rank  :
- * @factory: factory to rank
- * @rank: rank value - higher number means more priority rank
- *
- * Specifies a rank for the element so that 
- * autoplugging uses the most appropriate elements.
- *
- */
-void
-gst_element_factory_set_rank (GstElementFactory *factory, guint16 rank)
-{
-  g_return_if_fail (factory != NULL);
-  factory->rank = rank;
-}
-
 static void
 gst_element_factory_unload_thyself (GstPluginFeature *feature)
 {
@@ -421,5 +473,8 @@ gst_element_factory_unload_thyself (GstPluginFeature *feature)
 
   factory = GST_ELEMENT_FACTORY (feature);
 
-  factory->type = 0;
+  if (factory->type) {
+    g_type_class_unref (g_type_class_peek (factory->type));
+    factory->type = 0;
+  }
 }
