@@ -22,17 +22,25 @@
 
 #include <string.h>		/* memcpy */
 
-#include "gstinfo.h"
+#include "gst_private.h"
 #include "gstdata_private.h"
+
+#include "gstinfo.h"
+#include "gstmemchunk.h"
 #include "gstevent.h"
 #include "gstlog.h"
+#ifndef GST_DISABLE_TRACE
+/* #define GST_WITH_ALLOC_TRACE */
+#include "gsttrace.h"
+static GstAllocTrace *_event_trace;
+#endif
+
+static GstMemChunk *chunk;
 
 /* #define MEMPROF */
 
 GType _gst_event_type;
 
-static gint _gst_event_live;
-
 void
 _gst_event_initialize (void)
 {
@@ -40,29 +48,24 @@ _gst_event_initialize (void)
   _gst_event_type = g_boxed_type_register_static ("GstEvent",
                                                (GBoxedCopyFunc) gst_data_ref,
                                                (GBoxedFreeFunc) gst_data_unref);
-  _gst_event_live = 0;
-}
 
-/**
- * gst_event_print_stats:
- *
- * Logs statistics about live events (using g_log).
- */
-void
-gst_event_print_stats (void)
-{
-  g_log (g_log_domain_gstreamer, G_LOG_LEVEL_INFO,
-                    "%d live event(s)", _gst_event_live);
-}
+#ifndef GST_DISABLE_TRACE
+  _event_trace = gst_alloc_trace_register (GST_EVENT_TRACE_NAME);
+#endif
 
+  chunk = gst_mem_chunk_new ("GstEventChunk", sizeof (GstEvent),
+                                 sizeof (GstEvent) * 50, 0);
+}
 
 static GstEvent*
 _gst_event_copy (GstEvent *event)
 {
   GstEvent *copy;
 
-  copy = g_new0(GstEvent, 1);
-  _gst_event_live++;
+  copy = gst_mem_chunk_alloc (chunk);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_event_trace, copy);
+#endif
 
   memcpy (copy, event, sizeof (GstEvent));
   
@@ -74,7 +77,7 @@ _gst_event_copy (GstEvent *event)
 static void
 _gst_event_free (GstEvent* event)
 {
-  GST_INFO (GST_CAT_EVENT, "freeing event %p", event);
+  GST_CAT_INFO (GST_CAT_EVENT, "freeing event %p", event);
 
   if (GST_EVENT_SRC (event)) {
     gst_object_unref (GST_EVENT_SRC (event));
@@ -84,8 +87,10 @@ _gst_event_free (GstEvent* event)
       break;
   }
   _GST_DATA_DISPOSE (GST_DATA (event));
-  _gst_event_live--;
-  g_free (event);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_free (_event_trace, event);
+#endif
+  gst_mem_chunk_free (chunk, event);
 }
 
 /**
@@ -116,6 +121,12 @@ gst_event_masks_contains (const GstEventMask *masks, GstEventMask *mask)
   return FALSE;
 }
 
+GType
+gst_event_get_type (void)
+{
+  return _gst_event_type;
+}
+
 /**
  * gst_event_new:
  * @type: The type of the new event
@@ -129,9 +140,12 @@ gst_event_new (GstEventType type)
 {
   GstEvent *event;
 
-  event = g_new0(GstEvent, 1);
-  _gst_event_live++;
-  GST_INFO (GST_CAT_EVENT, "creating new event %p %d", event, type);
+  event = gst_mem_chunk_alloc0 (chunk);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_event_trace, event);
+#endif
+
+  GST_CAT_INFO (GST_CAT_EVENT, "creating new event %p %d", event, type);
 
   _GST_DATA_INIT (GST_DATA (event),
 		  _gst_event_type,
@@ -140,7 +154,7 @@ gst_event_new (GstEventType type)
 		  (GstDataCopyFunction) _gst_event_copy);
 
   GST_EVENT_TYPE (event) = type;
-  GST_EVENT_TIMESTAMP (event) = 0LL;
+  GST_EVENT_TIMESTAMP (event) = G_GINT64_CONSTANT (0);
   GST_EVENT_SRC (event) = NULL;
 
   return event;
@@ -169,13 +183,53 @@ gst_event_new_seek (GstSeekType type, gint64 offset)
   return event;
 }
 
+/**
+ * gst_event_new_discontinuousv:
+ * @new_media: A flag indicating a new media type starts
+ * @format1: The format of the discont value
+ * @var_args: more discont values and formats
+ *
+ * Allocate a new discontinuous event with the given format/value pairs. Note
+ * that the values are of type gint64 - you may not use simple integers such
+ * as "0" when calling this function, always cast them like "(gint64) 0".
+ * Terminate the list with #GST_FORMAT_UNDEFINED.
+ *
+ * Returns: A new discontinuous event.
+ */
+GstEvent*
+gst_event_new_discontinuousv (gboolean new_media, GstFormat format1, va_list var_args)
+{
+  GstEvent *event;
+  gint count = 0;
+
+  event = gst_event_new (GST_EVENT_DISCONTINUOUS);
+  GST_EVENT_DISCONT_NEW_MEDIA (event) = new_media;
+
+  while (format1 != GST_FORMAT_UNDEFINED && count < 8) {
+
+    GST_EVENT_DISCONT_OFFSET (event, count).format = format1 & GST_SEEK_FORMAT_MASK;
+    GST_EVENT_DISCONT_OFFSET (event, count).value = va_arg (var_args, gint64);
+
+    format1 = va_arg (var_args, GstFormat);
+
+    count++;
+  }
+
+  GST_EVENT_DISCONT_OFFSET_LEN (event) = count;
+		    
+  return event;
+}
+
 /**
  * gst_event_new_discontinuous:
  * @new_media: A flag indicating a new media type starts
  * @format1: The format of the discont value
  * @...: more discont values and formats
  *
- * Allocate a new discontinuous event with the geven format/value pairs.
+ * Allocate a new discontinuous event with the given format/value pairs. Note
+ * that the values are of type gint64 - you may not use simple integers such
+ * as "0" when calling this function, always cast them like "(gint64) 0".
+ * Terminate the list with #GST_FORMAT_UNDEFINED.
  *
  * Returns: A new discontinuous event.
  */
@@ -184,26 +238,13 @@ gst_event_new_discontinuous (gboolean new_media, GstFormat format1, ...)
 {
   va_list var_args;
   GstEvent *event;
-  gint count = 0;
-
-  event = gst_event_new (GST_EVENT_DISCONTINUOUS);
-  GST_EVENT_DISCONT_NEW_MEDIA (event) = new_media;
 
   va_start (var_args, format1);
-	        
-  while (format1) {
 
-    GST_EVENT_DISCONT_OFFSET (event, count).format = format1 & GST_SEEK_FORMAT_MASK;
-    GST_EVENT_DISCONT_OFFSET (event, count).value = va_arg (var_args, gint64);
+  event = gst_event_new_discontinuousv (new_media, format1, var_args);
 
-    format1 = va_arg (var_args, GstSeekType);
-
-    count++;
-  }
   va_end (var_args);
 
-  GST_EVENT_DISCONT_OFFSET_LEN (event) = count;
-		    
   return event;
 }
 
@@ -286,4 +327,3 @@ gst_event_new_segment_seek (GstSeekType type, gint64 start, gint64 stop)
 
   return event;
 }
-
