@@ -20,10 +20,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "gstlog.h"
 #include "gst_private.h"
-#include "gstregistry.h"
 
+#include "gstinfo.h"
+#include "gstregistrypool.h"
 #include "gstpad.h"
 #include "gstindex.h"
 
@@ -89,7 +89,21 @@ gst_index_resolver_get_type (void)
 }
 
 GType
-gst_index_get_type(void) {
+gst_index_entry_get_type (void)
+{
+  static GType index_entry_type = 0;
+
+  if (!index_entry_type) {
+    index_entry_type = g_boxed_type_register_static ("GstIndexEntry",
+        (GBoxedCopyFunc) gst_index_entry_copy,
+        (GBoxedFreeFunc) gst_index_entry_free);
+  }
+  return index_entry_type;
+}
+
+
+GType
+gst_index_get_type (void) {
   static GType index_type = 0;
 
   if (!index_type) {
@@ -152,7 +166,7 @@ gst_index_init (GstIndex *index)
   GST_FLAG_SET (index, GST_INDEX_WRITABLE);
   GST_FLAG_SET (index, GST_INDEX_READABLE);
   
-  GST_DEBUG(0, "created new index");
+  GST_DEBUG ( "created new index");
 }
 
 static void
@@ -203,7 +217,7 @@ gst_index_group_new(guint groupnum)
   indexgroup->certainty = GST_INDEX_UNKNOWN;
   indexgroup->peergroup = -1;
 
-  GST_DEBUG(0, "created new index group %d",groupnum);
+  GST_DEBUG ( "created new index group %d",groupnum);
 
   return indexgroup;
 }
@@ -216,7 +230,7 @@ gst_index_group_new(guint groupnum)
  * Returns: a new index object
  */
 GstIndex*
-gst_index_new()
+gst_index_new (void)
 {
   GstIndex *index;
 
@@ -274,7 +288,7 @@ gst_index_new_group(GstIndex *index)
 {
   index->curgroup = gst_index_group_new(++index->maxgroup);
   index->groups = g_list_append(index->groups,index->curgroup);
-  GST_DEBUG(0, "created new group %d in index",index->maxgroup);
+  GST_DEBUG ( "created new group %d in index",index->maxgroup);
   return index->maxgroup;
 }
 
@@ -305,13 +319,13 @@ gst_index_set_group(GstIndex *index, gint groupnum)
     list = g_list_next(list);
     if (indexgroup->groupnum == groupnum) {
       index->curgroup = indexgroup;
-      GST_DEBUG(0, "switched to index group %d", indexgroup->groupnum);
+      GST_DEBUG ( "switched to index group %d", indexgroup->groupnum);
       return TRUE;
     }
   }
 
   /* couldn't find the group in question */
-  GST_DEBUG(0, "couldn't find index group %d",groupnum);
+  GST_DEBUG ( "couldn't find index group %d",groupnum);
   return FALSE;
 }
 
@@ -381,6 +395,18 @@ gst_index_set_resolver (GstIndex *index,
   index->method = GST_INDEX_RESOLVER_CUSTOM;
 }
 
+/**
+ * gst_index_entry_copy:
+ * @entry: the entry to copy
+ *
+ * Copies an entry and returns the result.
+ */
+GstIndexEntry *
+gst_index_entry_copy (GstIndexEntry *entry)
+{
+  return g_memdup(entry, sizeof(*entry));
+}
+
 /**
  * gst_index_entry_free:
  * @entry: the entry to free
@@ -555,7 +581,7 @@ gst_index_get_writer_id (GstIndex *index, GstObject *writer, gint *id)
     }
     /* if the index could not resolve, we allocate one ourselves */
     if (!success) {
-      *id = index->last_id++;
+      *id = ++index->last_id;
     }
 
     entry = gst_index_add_id (index, *id, writer_string);
@@ -892,7 +918,7 @@ gst_index_factory_find (const gchar *name)
 
   g_return_val_if_fail (name != NULL, NULL);
 
-  GST_DEBUG (0,"gstindex: find \"%s\"", name);
+  GST_DEBUG ("gstindex: find \"%s\"", name);
 
   feature = gst_registry_pool_find_feature (name, GST_TYPE_INDEX_FACTORY);
   if (feature)
@@ -949,4 +975,3 @@ gst_index_factory_make (const gchar *name)
 
   return gst_index_factory_create (factory);
 }
-
