@@ -25,7 +25,6 @@
 #include "gstpad.h"
 #include "gstutils.h"
 #include "gstelement.h"
-#include "gsttype.h"
 #include "gstbin.h"
 #include "gstscheduler.h"
 #include "gstevent.h"
@@ -171,17 +170,17 @@ gst_real_pad_class_init (GstRealPadClass *klass)
     g_signal_new ("caps_nego_failed", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GstRealPadClass, caps_nego_failed), NULL, NULL,
                   gst_marshal_VOID__POINTER, G_TYPE_NONE, 1,
-                  G_TYPE_POINTER);
+                  GST_TYPE_CAPS);
   gst_real_pad_signals[REAL_LINKED] =
     g_signal_new ("linked", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GstRealPadClass, linked), NULL, NULL,
                   gst_marshal_VOID__POINTER, G_TYPE_NONE, 1,
-                  G_TYPE_POINTER);
+                  GST_TYPE_PAD);
   gst_real_pad_signals[REAL_UNLINKED] =
     g_signal_new ("unlinked", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GstRealPadClass, unlinked), NULL, NULL,
                   gst_marshal_VOID__POINTER, G_TYPE_NONE, 1,
-                  G_TYPE_POINTER);
+                  GST_TYPE_PAD);
 
 /*  gtk_object_add_arg_type ("GstRealPad::active", G_TYPE_BOOLEAN, */
 /*                           GTK_ARG_READWRITE, REAL_ARG_ACTIVE); */
@@ -217,7 +216,6 @@ gst_real_pad_init (GstRealPad *pad)
   pad->linkfunc = NULL;
   pad->getcapsfunc = NULL;
 
-  pad->convertfunc 	= gst_pad_convert_default;
   pad->eventfunc 	= gst_pad_event_default;
   pad->convertfunc 	= gst_pad_convert_default;
   pad->queryfunc 	= gst_pad_query_default;
@@ -864,11 +862,11 @@ gst_pad_unlink (GstPad *srcpad,
   g_return_if_fail ((GST_RPAD_DIRECTION (realsrc) == GST_PAD_SRC) &&
                     (GST_RPAD_DIRECTION (realsink) == GST_PAD_SINK));
 
-  if (GST_RPAD_UNLINKFUNC (srcpad)) {
-    GST_RPAD_UNLINKFUNC (srcpad) (srcpad);
+  if (GST_RPAD_UNLINKFUNC (realsrc)) {
+    GST_RPAD_UNLINKFUNC (realsrc) (GST_PAD_CAST (realsrc));
   }
-  if (GST_RPAD_UNLINKFUNC (sinkpad)) {
-    GST_RPAD_UNLINKFUNC (sinkpad) (sinkpad);
+  if (GST_RPAD_UNLINKFUNC (realsink)) {
+    GST_RPAD_UNLINKFUNC (realsink) (GST_PAD_CAST (realsink));
   }
 
   /* get the schedulers before we unlink */
@@ -2311,14 +2309,19 @@ gst_pad_push (GstPad *pad, GstData *data)
 {
   GstRealPad *peer;
 
-  g_assert (GST_IS_PAD (pad));
-  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, pad, "pushing");
-
+  g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (GST_PAD_DIRECTION (pad) == GST_PAD_SRC);
 
   if (!gst_probe_dispatcher_dispatch (&(GST_REAL_PAD (pad)->probedisp), &data))
     return;
+  
+  if (!GST_PAD_IS_LINKED (pad)) {
+    GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, pad, "not pushing data %p as pad is unconnected", data);
+    gst_data_unref (data);
+    return;
+  }      
 
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, pad, "pushing");
   peer = GST_RPAD_PEER (pad);
 
   if (!peer) {
@@ -2904,7 +2907,7 @@ gst_pad_event_default_dispatch (GstPad *pad, GstElement *element,
   }
   gst_event_unref (event);
   g_list_free (orig);
-  return TRUE;
+  return (GST_PAD_DIRECTION (pad) == GST_PAD_SINK);
 }
 
 /**
