@@ -133,8 +133,8 @@ typedef enum {
 /* this defines the functions used to chain buffers
  * pad is the sink pad (so the same chain function can be used for N pads)
  * buf is the buffer being passed */
-typedef void 			(*GstPadChainFunction) 		(GstPad *pad,GstBuffer *buf);
-typedef GstBuffer*		(*GstPadGetFunction) 		(GstPad *pad);
+typedef void 			(*GstPadChainFunction) 		(GstPad *pad,GstData *data);
+typedef GstData*		(*GstPadGetFunction) 		(GstPad *pad);
 typedef gboolean		(*GstPadEventFunction)		(GstPad *pad, GstEvent *event);
 typedef gboolean		(*GstPadConvertFunction)	(GstPad *pad, 
 		 						 GstFormat src_format,  gint64  src_value,
@@ -172,10 +172,14 @@ struct _GstPad {
   gpointer 		element_private;
 
   GstPadTemplate 	*padtemplate;	/* the template for this pad */
+
+  GST_OBJECT_PADDING
 };
 
 struct _GstPadClass {
   GstObjectClass parent_class;
+
+  GST_CLASS_PADDING
 };
 
 struct _GstRealPad {
@@ -216,25 +220,34 @@ struct _GstRealPad {
   GstPadBufferPoolFunction 	 bufferpoolfunc;
 
   GstProbeDispatcher 		 probedisp;
+
+  GST_OBJECT_PADDING
 };
 
 struct _GstRealPadClass {
-  GstPadClass parent_class;
+  GstPadClass 	parent_class;
 
   /* signal callbacks */
-  void (*caps_nego_failed)	(GstPad *pad);
-  void (*linked)		(GstPad *pad, GstPad *peer);
-  void (*unlinked)		(GstPad *pad, GstPad *peer);
+  void		(*caps_nego_failed)	(GstPad *pad, GstCaps *caps);
+
+  void 		(*linked)		(GstPad *pad, GstPad *peer);
+  void 		(*unlinked)		(GstPad *pad, GstPad *peer);
+
+  GST_CLASS_PADDING
 };
 
 struct _GstGhostPad {
   GstPad pad;
 
   GstRealPad *realpad;
+
+  GST_OBJECT_PADDING
 };
 
 struct _GstGhostPadClass {
   GstPadClass parent_class;
+
+  GST_CLASS_PADDING
 };
 
 
@@ -306,12 +319,12 @@ typedef enum {
 #define GST_PAD_TEMPLATE_CAPS(templ)		(((GstPadTemplate *)(templ))->caps)
 
 typedef enum {
-  GST_PAD_TEMPLATE_FIXED        = GST_OBJECT_FLAG_LAST,
-	    
-  GST_PAD_TEMPLATE_FLAG_LAST    = GST_OBJECT_FLAG_LAST + 4
+  GST_PAD_TEMPLATE_FIXED	= GST_OBJECT_FLAG_LAST,
+
+  GST_PAD_TEMPLATE_FLAG_LAST	= GST_OBJECT_FLAG_LAST + 4
 } GstPadTemplateFlags;
 
-#define GST_PAD_TEMPLATE_IS_FIXED(templ)        (GST_FLAG_IS_SET(templ), GST_PAD_TEMPLATE_FIXED)
+#define GST_PAD_TEMPLATE_IS_FIXED(templ)	(GST_FLAG_IS_SET(templ, GST_PAD_TEMPLATE_FIXED))
 
 struct _GstPadTemplate {
   GstObject	   object;
@@ -320,6 +333,8 @@ struct _GstPadTemplate {
   GstPadDirection  direction;
   GstPadPresence   presence;
   GstCaps	  *caps;
+
+  GST_OBJECT_PADDING
 };
 
 struct _GstPadTemplateClass {
@@ -327,6 +342,8 @@ struct _GstPadTemplateClass {
 
   /* signal callbacks */
   void (*pad_created)	(GstPadTemplate *templ, GstPad *pad);
+
+  GST_CLASS_PADDING
 };
 
 #ifdef G_HAVE_ISO_VARARGS
@@ -350,7 +367,7 @@ name (void)                                     \
       pres,                                     \
       __VA_ARGS__ );                            \
   }                                             \
-  return templ;                              	\
+  return (GstPadTemplate *)g_object_ref((GObject *)templ); \
 }
 #elif defined(G_HAVE_GNUC_VARARGS)
 /* CR1: the space after 'a' is necessary because of preprocessing in gcc */
@@ -374,7 +391,7 @@ name (void)                                     \
       pres,                                     \
       a );                                      \
   }                                             \
-  return templ;                              	\
+  return (GstPadTemplate *)g_object_ref((GObject *)templ); \
 }
 #endif
 
@@ -386,7 +403,6 @@ GType			gst_ghost_pad_get_type			(void);
 
 /* creating pads */
 GstPad*			gst_pad_new				(const gchar *name, GstPadDirection direction);
-#define			gst_pad_destroy(pad)			gst_object_destroy (GST_OBJECT (pad))
 GstPad*			gst_pad_new_from_template		(GstPadTemplate *templ, const gchar *name);
 GstPad*			gst_pad_custom_new			(GType type, const gchar *name, GstPadDirection direction);
 GstPad*			gst_pad_custom_new_from_template	(GType type, GstPadTemplate *templ, const gchar *name);
@@ -429,7 +445,7 @@ const GstEventMask*	gst_pad_get_event_masks_default		(GstPad *pad);
 void			gst_pad_set_link_function		(GstPad *pad, GstPadLinkFunction link);
 gboolean                gst_pad_can_link            		(GstPad *srcpad, GstPad *sinkpad);
 gboolean                gst_pad_can_link_filtered   		(GstPad *srcpad, GstPad *sinkpad, GstCaps *filtercaps);
-void			gst_pad_set_unlink_function		(GstPad *pad, GstPadUnlinkFunction link);
+void			gst_pad_set_unlink_function		(GstPad *pad, GstPadUnlinkFunction unlink);
 
 gboolean                gst_pad_link             		(GstPad *srcpad, GstPad *sinkpad);
 gboolean                gst_pad_link_filtered       		(GstPad *srcpad, GstPad *sinkpad, GstCaps *filtercaps);
@@ -451,9 +467,11 @@ gboolean		gst_pad_try_relink_filtered		(GstPad *srcpad, GstPad *sinkpad, GstCaps
 GstCaps*	     	gst_pad_get_allowed_caps       		(GstPad *pad);
 gboolean	     	gst_pad_recalc_allowed_caps    		(GstPad *pad);
 
+gboolean	     	gst_pad_recover_caps_error    		(GstPad *pad, GstCaps *allowed);
+
 /* data passing functions */
-void			gst_pad_push				(GstPad *pad, GstBuffer *buf);
-GstBuffer*		gst_pad_pull				(GstPad *pad);
+void			gst_pad_push				(GstPad *pad, GstData *data);
+GstData*		gst_pad_pull				(GstPad *pad);
 gboolean		gst_pad_send_event			(GstPad *pad, GstEvent *event);
 gboolean		gst_pad_event_default			(GstPad *pad, GstEvent *event);
 GstPad*			gst_pad_select				(GList *padlist);
@@ -492,9 +510,9 @@ gboolean 		gst_pad_dispatcher 			(GstPad *pad, GstPadDispatcherFunction dispatch
 								 gpointer data);
 
 #define			gst_pad_add_probe(pad, probe) \
-			(gst_probe_dispatcher_add_probe (&(GST_REAL_PAD (pad)-probedisp), probe))
+			(gst_probe_dispatcher_add_probe (&(GST_REAL_PAD (pad)->probedisp), probe))
 #define			gst_pad_remove_probe(pad, probe) \
-			(gst_probe_dispatcher_remove_probe (&(GST_REAL_PAD (pad)-probedisp), probe))
+			(gst_probe_dispatcher_remove_probe (&(GST_REAL_PAD (pad)->probedisp), probe))
 
 #ifndef GST_DISABLE_LOADSAVE
 void			gst_pad_load_and_link			(xmlNodePtr self, GstObject *parent);
@@ -512,11 +530,17 @@ GstPadTemplate*		gst_pad_template_new			(const gchar *name_template,
 		                                        	 GstPadDirection direction, GstPadPresence presence,
 								 GstCaps *caps, ...);
 
+GstPadTemplate*		gst_pad_template_newv			(const gchar *name_template,
+		                                        	 GstPadDirection direction, GstPadPresence presence,
+								 GstCaps *caps, va_list var_args);
+
 GstCaps*		gst_pad_template_get_caps		(GstPadTemplate *templ);
 GstCaps*		gst_pad_template_get_caps_by_name	(GstPadTemplate *templ, const gchar *name);
 
+#ifndef GST_DISABLE_LOADSAVE
 xmlNodePtr              gst_ghost_pad_save_thyself   		(GstPad *pad,
 						     		 xmlNodePtr parent);
+#endif
 
 G_END_DECLS
 
