@@ -27,6 +27,8 @@
 #include "gstregistry.h"
 #include "gstinfo.h"
 
+#include <string.h>
+
 static void		gst_plugin_feature_class_init		(GstPluginFeatureClass *klass);
 static void		gst_plugin_feature_init			(GstPluginFeature *feature);
 
@@ -96,7 +98,7 @@ gst_plugin_feature_ensure_loaded (GstPluginFeature *feature)
 #ifndef GST_DISABLE_REGISTRY 
     if (GST_IS_REGISTRY (plugin->manager)) {
       GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, 
-	         "loading plugin %s for feature", plugin->name);
+	         "loading plugin %s for feature", plugin->desc.name);
 
       if (gst_registry_load_plugin (GST_REGISTRY (plugin->manager), plugin) != GST_REGISTRY_OK)
 	return FALSE;
@@ -136,7 +138,6 @@ gst_plugin_feature_type_name_filter (GstPluginFeature *feature,
   return ((data->type == 0    || data->type == G_OBJECT_TYPE (feature)) &&
           (data->name == NULL || !strcmp (data->name, GST_PLUGIN_FEATURE_NAME (feature))));
 }
-
 /**
  * gst_plugin_feature_set_rank:
  * @feature: feature to rank
@@ -153,4 +154,25 @@ gst_plugin_feature_set_rank (GstPluginFeature *feature, guint16 rank)
 
   feature->rank = rank;
 }
+/**
+ * gst_plugin_feature_set_rank:
+ * @feature: a feature
+ * @name: the name to set
+ *
+ * Sets the name of a plugin feature. The name uniquely identifies a feature
+ * within all features of the same type. Renaming a plugin feature is not 
+ * allowed.
+ */
+void
+gst_plugin_feature_set_name (GstPluginFeature *feature, const gchar *name)
+{
+  g_return_if_fail (GST_IS_PLUGIN_FEATURE (feature));
+  g_return_if_fail (name != NULL);
+
+  if (feature->name) {
+    g_return_if_fail (strcmp (feature->name, name) == 0);
+  } else {
+    feature->name = g_strdup (name);
+  }
+}
 
