@@ -29,22 +29,11 @@
 
 G_BEGIN_DECLS
 
-gboolean        gst_util_has_arg                (GObject *object, const gchar *argname,
-						 GType    arg_type);
-gint 		gst_util_get_int_arg		(GObject *object, const gchar *argname);
-gboolean	gst_util_get_bool_arg		(GObject *object, const gchar *argname);
-glong 		gst_util_get_long_arg		(GObject *object, const gchar *argname);
-gint64         gst_util_get_int64_arg		(GObject *object, const gchar *argname);
-gfloat 		gst_util_get_float_arg		(GObject *object, const gchar *argname);
-gdouble 	gst_util_get_double_arg		(GObject *object, const gchar *argname);
-const gchar*	gst_util_get_string_arg		(GObject *object, const gchar *argname);
-gpointer 	gst_util_get_pointer_arg	(GObject *object, const gchar *argname);
 void		gst_util_set_value_from_string	(GValue *value, const gchar *value_str);
 void 		gst_util_set_object_arg 	(GObject *object, const gchar *name, const gchar *value);
 	
 void 		gst_util_dump_mem		(guchar *mem, guint size);
 
-
 void 		gst_print_pad_caps 		(GString *buf, gint indent, GstPad *pad);
 void 		gst_print_element_args 		(GString *buf, gint indent, GstElement *element);
 
