@@ -48,14 +48,20 @@ struct _GstXML {
   GList      *topelements;
 
   xmlNsPtr ns;
+
+  GST_OBJECT_PADDING
 };
 
+typedef struct _GstXMLNs GstXMLNs;
+
 struct _GstXMLClass {
   GstObjectClass parent_class;
 
   /* signal callbacks */
   void (*object_loaded)         (GstXML *xml, GstObject *object, xmlNodePtr self);
   void (*object_saved)          (GstXML *xml, GstObject *object, xmlNodePtr self);
+
+  GST_CLASS_PADDING
 };
 
 GType		gst_xml_get_type	(void);
