@@ -100,21 +100,15 @@ static void do_cothread_switch(cothread *to)
 {
   cothread *from = cothread_self ();
   if (from == (to)) {
-    GST_DEBUG (GST_CAT_COTHREAD_SWITCH,
-               "trying to switch to the same cothread (%p), not allowed",
-               (to));
     g_warning ("trying to switch to the same cothread, not allowed");
   } else {
-    GST_INFO (GST_CAT_COTHREAD_SWITCH,
-              "switching from cothread %p to cothread %p", from, (to));
     cothread_switch (from, (to));
-    GST_INFO (GST_CAT_COTHREAD_SWITCH, "we're in cothread %p now", from);
   }
 }
 
 #define do_cothread_create(new_cothread, context, func, argc, argv)         \
   G_STMT_START{                                                             \
-    new_cothread = cothread_create ((func), 0, (char **) (argv), (context)); \
+    new_cothread = cothread_create ((func), 0, (char**) (argv), (context)); \
   }G_STMT_END
   
 #define do_cothread_setfunc(cothread, context, func, argc, argv)            \
