@@ -19,39 +19,52 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include <gst/gstinfo.h>
+#include <gst/gstplugin.h>
+#include <gst/gstversion.h>
 #include "bytestream.h"
 
-/* #define BS_DEBUG */
-
-#ifdef G_HAVE_ISO_VARARGS
-
-#ifdef BS_DEBUG
-# define bs_print(...)	GST_DEBUG (GST_CAT_BUFFER, __VA_ARGS__)
-# define bs_status(bs)			gst_bytestream_print_status(bs)
-#else
-# define bs_print(...)
-# define bs_status(bs)
-#endif
-
-#elif defined(G_HAVE_GNUC_VARARGS)
-
-#ifdef BS_DEBUG
-# define bs_print(format,args...)	GST_DEBUG (GST_CAT_BUFFER,  format, ## args)
-# define bs_status(bs)			gst_bytestream_print_status(bs)
-#else
-# define bs_print(format,args...)
-# define bs_status(bs)
-#endif
-
-#endif
+GST_DEBUG_CATEGORY_STATIC(debug_bs);
+#define GST_CAT_DEFAULT debug_bs
 
 static guint8 *gst_bytestream_assemble (GstByteStream * bs, guint32 len);
 
+static inline void
+gst_bytestream_init (GstByteStream *bs)
+{
+  bs->event = NULL;
+  bs->buflist = NULL;
+  bs->headbufavail = 0;
+  bs->listavail = 0;
+  bs->assembled = NULL;
+  bs->offset = 0LL;
+  bs->in_seek = FALSE;
+}
+static inline void
+gst_bytestream_exit (GstByteStream *bs)
+{
+  GSList *walk;
+
+  if (bs->event)
+    gst_event_unref (bs->event);
+  
+  walk = bs->buflist;
+  while (walk) {
+    gst_buffer_unref (GST_BUFFER (walk->data));
+    walk = g_slist_next (walk);
+  }
+  g_slist_free (bs->buflist);
+
+  g_free (bs->assembled);
+}
 /**
  * gst_bytestream_new:
  * @pad: the pad to attach the bytestream to
@@ -66,36 +79,31 @@ gst_bytestream_new (GstPad * pad)
   GstByteStream *bs = g_new (GstByteStream, 1);
 
   bs->pad = pad;
-  bs->event = NULL;
-  bs->buflist = NULL;
-  bs->headbufavail = 0;
-  bs->listavail = 0;
-  bs->assembled = NULL;
-  bs->offset = 0LL;
-  bs->in_seek = FALSE;
+  gst_bytestream_init (bs);
 
   return bs;
 }
 
+/**
+ * gst_bytestream_destroy:
+ * @bs: the bytestream object to destroy
+ *
+ * destroy the bytestream object and free its resources.
+ */
 void
 gst_bytestream_destroy (GstByteStream * bs)
 {
-  GSList *walk;
-
-  if (bs->event)
-    gst_event_unref (bs->event);
-
-  walk = bs->buflist;
-  while (walk) {
-    gst_buffer_unref (GST_BUFFER (walk->data));
-    walk = g_slist_next (walk);
-  }
-  g_slist_free (bs->buflist);
-  if (bs->assembled)
-    g_free (bs->assembled);
+  gst_bytestream_exit (bs);
   g_free (bs);
 }
-
+void
+gst_bytestream_reset (GstByteStream *bs)
+{
+  /* free all data */
+  gst_bytestream_exit (bs);
+  /* reset data to clean state */
+  gst_bytestream_init (bs);
+}
 /* HOW THIS WORKS:
  *
  * The fundamental structure is a singly-linked list of buffers.  The
@@ -136,14 +144,13 @@ gst_bytestream_get_next_buf (GstByteStream *bs)
 {
   GstBuffer *nextbuf, *lastbuf, *headbuf;
   GSList *end;
-  GstClockTime ts;
 
   /* if there is an event pending, return FALSE */
   if (bs->event)
     return FALSE;
 
-  bs_print ("get_next_buf: pulling buffer");
-  nextbuf = gst_pad_pull (bs->pad);
+  GST_DEBUG ("get_next_buf: pulling buffer");
+  nextbuf = GST_BUFFER (gst_pad_pull (bs->pad));
 
   if (!nextbuf)
     return FALSE;
@@ -153,15 +160,14 @@ gst_bytestream_get_next_buf (GstByteStream *bs)
     return FALSE;
   }
 
-  ts = GST_BUFFER_TIMESTAMP (nextbuf);
-  if (ts != GST_CLOCK_TIME_NONE)
-    bs->last_ts = ts;
+  if (GST_BUFFER_TIMESTAMP_IS_VALID (nextbuf))
+    bs->last_ts = GST_BUFFER_TIMESTAMP (nextbuf);
 
-  bs_print ("get_next_buf: got buffer of %d bytes", GST_BUFFER_SIZE (nextbuf));
+  GST_DEBUG ("get_next_buf: got buffer of %d bytes", GST_BUFFER_SIZE (nextbuf));
 
   /* first see if there are any buffers in the list at all */
   if (bs->buflist) {
-    bs_print ("gst_next_buf: there is at least one buffer in the list");
+    GST_DEBUG ("gst_next_buf: there is at least one buffer in the list");
     /* now find the end of the list */
     end = g_slist_last (bs->buflist);
     /* get the buffer that's there */
@@ -169,7 +175,7 @@ gst_bytestream_get_next_buf (GstByteStream *bs)
 
     /* see if we can marge cheaply */
     if (gst_buffer_is_span_fast (lastbuf, nextbuf)) {
-      bs_print ("get_next_buf: merging new buffer with last buf on list");
+      GST_DEBUG ("get_next_buf: merging new buffer with last buf on list");
       /* it is, let's merge them (this is really an append, but...) */
       end->data = gst_buffer_merge (lastbuf, nextbuf);
       /* add to the length of the list */
@@ -186,7 +192,7 @@ gst_bytestream_get_next_buf (GstByteStream *bs)
       /* if we can't, we just append this buffer */
     }
     else {
-      bs_print ("get_next_buf: adding new buffer to the end of the list");
+      GST_DEBUG ("get_next_buf: adding new buffer to the end of the list");
       end = g_slist_append (end, nextbuf);
       /* also need to increment length of list and buffer count */
       bs->listavail += GST_BUFFER_SIZE (nextbuf);
@@ -195,7 +201,7 @@ gst_bytestream_get_next_buf (GstByteStream *bs)
     /* if there are no buffers in the list */
   }
   else {
-    bs_print ("get_next_buf: buflist is empty, adding new buffer to list");
+    GST_DEBUG ("get_next_buf: buflist is empty, adding new buffer to list");
     /* put this on the end of the list */
     bs->buflist = g_slist_append (bs->buflist, nextbuf);
     /* and increment the number of bytes in the list */
@@ -218,7 +224,7 @@ gst_bytestream_fill_bytes (GstByteStream *bs, guint32 len)
 {
   /* as long as we don't have enough, we get more buffers */
   while (bs->listavail < len) {
-    bs_print ("fill_bytes: there are %d bytes in the list, we need %d", bs->listavail, len);
+    GST_DEBUG ("fill_bytes: there are %d bytes in the list, we need %d", bs->listavail, len);
     if (!gst_bytestream_get_next_buf (bs))
       return FALSE;
   }
@@ -226,6 +232,20 @@ gst_bytestream_fill_bytes (GstByteStream *bs, guint32 len)
   return TRUE;
 }
 
+/**
+ * gst_bytestream_peek:
+ * @bs: the bytestream to peek
+ * @buf: pointer to a variable that can hold a buffer pointer.
+ * @len: the number of bytes to peek
+ *
+ * Peeks len bytes into the bytestream, the result is returned as
+ * a #GstBuffer. Unref the buffer after usage.
+ * This function can return less bytes than requested. In that case,
+ * an event might have happened which you can retrieve with
+ * gst_bytestream_get_status().
+ *
+ * Returns: The number of bytes successfully peeked.
+ */
 guint32
 gst_bytestream_peek (GstByteStream *bs, GstBuffer **buf, guint32 len)
 {
@@ -234,10 +254,10 @@ gst_bytestream_peek (GstByteStream *bs, GstBuffer **buf, guint32 len)
   g_return_val_if_fail (bs != NULL, 0);
   g_return_val_if_fail (len > 0, 0);
 
-  bs_print ("peek: asking for %d bytes", len);
+  GST_DEBUG ("peek: asking for %d bytes", len);
 
   /* make sure we have enough */
-  bs_print ("peek: there are %d bytes in the list", bs->listavail);
+  GST_DEBUG ("peek: there are %d bytes in the list", bs->listavail);
   if (len > bs->listavail) {
     if (!gst_bytestream_fill_bytes (bs, len)) {
       /* we must have an event coming up */
@@ -251,37 +271,52 @@ gst_bytestream_peek (GstByteStream *bs, GstBuffer **buf, guint32 len)
         return 0;
       }
     }
-    bs_print ("peek: there are now %d bytes in the list", bs->listavail);
+    GST_DEBUG ("peek: there are now %d bytes in the list", bs->listavail);
   }
-  bs_status (bs);
+  gst_bytestream_print_status (bs);
 
   /* extract the head buffer */
   headbuf = GST_BUFFER (bs->buflist->data);
 
   /* if the requested bytes are in the current buffer */
-  bs_print ("peek: headbufavail is %d", bs->headbufavail);
+  GST_DEBUG ("peek: headbufavail is %d", bs->headbufavail);
   if (len <= bs->headbufavail) {
-    bs_print ("peek: there are enough bytes in headbuf (need %d, have %d)", len, bs->headbufavail);
+    GST_DEBUG ("peek: there are enough bytes in headbuf (need %d, have %d)", len, bs->headbufavail);
     /* create a sub-buffer of the headbuf */
     retbuf = gst_buffer_create_sub (headbuf, GST_BUFFER_SIZE (headbuf) - bs->headbufavail, len);
+    GST_BUFFER_OFFSET (retbuf) = GST_BUFFER_OFFSET (headbuf) + GST_BUFFER_SIZE (headbuf) - bs->headbufavail;
 
-    /* otherwise we need to figure out how to assemble one */
   }
+  /* otherwise we need to figure out how to assemble one */
   else {
-    bs_print ("peek: current buffer is not big enough for len %d", len);
+    GST_DEBUG ("peek: current buffer is not big enough for len %d", len);
 
     retbuf = gst_buffer_new ();
     GST_BUFFER_SIZE (retbuf) = len;
     GST_BUFFER_DATA (retbuf) = gst_bytestream_assemble (bs, len);
     GST_BUFFER_TIMESTAMP (retbuf) = bs->last_ts;
-    if (GST_BUFFER_OFFSET (headbuf) != -1)
-      GST_BUFFER_OFFSET (retbuf) = GST_BUFFER_OFFSET (headbuf) + (GST_BUFFER_SIZE (headbuf) - bs->headbufavail);
   }
 
   *buf = retbuf;
   return len;
 }
 
+/**
+ * gst_bytestream_peek_bytes:
+ * @bs: the bytestream to peek
+ * @data: pointer to a variable that can hold a guint8 pointer.
+ * @len: the number of bytes to peek
+ *
+ * Peeks len bytes into the bytestream, the result is returned as
+ * a pointer to a guint8*. The data pointed to be data should not
+ * be freed and will become invalid after performing the next bytestream
+ * operation.
+ * This function can return less bytes than requested. In that case,
+ * an event might have happened which you can retrieve with
+ * gst_bytestream_get_status().
+ *
+ * Returns: The number of bytes successfully peeked.
+ */
 guint32
 gst_bytestream_peek_bytes (GstByteStream *bs, guint8** data, guint32 len)
 {
@@ -290,14 +325,18 @@ gst_bytestream_peek_bytes (GstByteStream *bs, guint8** data, guint32 len)
   g_return_val_if_fail (bs != NULL, 0);
   g_return_val_if_fail (len > 0, 0);
 
-  bs_print ("peek_bytes: asking for %d bytes", len);
+  GST_DEBUG ("peek_bytes: asking for %d bytes", len);
   if (bs->assembled) {
+    if (bs->assembled_len >= len) {
+      *data = bs->assembled;
+      return len;
+    }
     g_free (bs->assembled);
     bs->assembled = NULL;
   }
 
   /* make sure we have enough */
-  bs_print ("peek_bytes: there are %d bytes in the list", bs->listavail);
+  GST_DEBUG ("peek_bytes: there are %d bytes in the list", bs->listavail);
   if (len > bs->listavail) {
     if (!gst_bytestream_fill_bytes (bs, len)){
       /* we must have an event coming up */
@@ -311,24 +350,24 @@ gst_bytestream_peek_bytes (GstByteStream *bs, guint8** data, guint32 len)
         return 0;
       }
     }
-    bs_print ("peek_bytes: there are now %d bytes in the list", bs->listavail);
+    GST_DEBUG ("peek_bytes: there are now %d bytes in the list", bs->listavail);
   }
-  bs_status (bs);
+  gst_bytestream_print_status (bs);
 
   /* extract the head buffer */
   headbuf = GST_BUFFER (bs->buflist->data);
 
   /* if the requested bytes are in the current buffer */
-  bs_print ("peek_bytes: headbufavail is %d", bs->headbufavail);
+  GST_DEBUG ("peek_bytes: headbufavail is %d", bs->headbufavail);
   if (len <= bs->headbufavail) {
-    bs_print ("peek_bytes: there are enough bytes in headbuf (need %d, have %d)", len, bs->headbufavail);
+    GST_DEBUG ("peek_bytes: there are enough bytes in headbuf (need %d, have %d)", len, bs->headbufavail);
     /* create a sub-buffer of the headbuf */
     *data = GST_BUFFER_DATA (headbuf) + (GST_BUFFER_SIZE (headbuf) - bs->headbufavail);
 
-    /* otherwise we need to figure out how to assemble one */
   }
+  /* otherwise we need to figure out how to assemble one */
   else {
-    bs_print ("peek_bytes: current buffer is not big enough for len %d", len);
+    GST_DEBUG ("peek_bytes: current buffer is not big enough for len %d", len);
 
     *data = gst_bytestream_assemble (bs, len);
     bs->assembled = *data;
@@ -348,7 +387,7 @@ gst_bytestream_assemble (GstByteStream *bs, guint32 len)
 
   /* copy the data from the curbuf */
   buf = GST_BUFFER (bs->buflist->data);
-  bs_print ("assemble: copying %d bytes from curbuf at %d to *data", bs->headbufavail,
+  GST_DEBUG ("assemble: copying %d bytes from curbuf at %d to *data", bs->headbufavail,
 	    GST_BUFFER_SIZE (buf) - bs->headbufavail);
   memcpy (data, GST_BUFFER_DATA (buf) + GST_BUFFER_SIZE (buf) - bs->headbufavail, bs->headbufavail);
   copied += bs->headbufavail;
@@ -358,12 +397,12 @@ gst_bytestream_assemble (GstByteStream *bs, guint32 len)
   while (copied < len) {
     buf = GST_BUFFER (walk->data);
     if (GST_BUFFER_SIZE (buf) < (len - copied)) {
-      bs_print ("assemble: copying %d bytes from buf to output offset %d", GST_BUFFER_SIZE (buf), copied);
+      GST_DEBUG ("assemble: copying %d bytes from buf to output offset %d", GST_BUFFER_SIZE (buf), copied);
       memcpy (data + copied, GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
       copied += GST_BUFFER_SIZE (buf);
     }
     else {
-      bs_print ("assemble: copying %d bytes from buf to output offset %d", len - copied, copied);
+      GST_DEBUG ("assemble: copying %d bytes from buf to output offset %d", len - copied, copied);
       memcpy (data + copied, GST_BUFFER_DATA (buf), len - copied);
       copied = len;
     }
@@ -373,20 +412,34 @@ gst_bytestream_assemble (GstByteStream *bs, guint32 len)
   return data;
 }
 
+/**
+ * gst_bytestream_flush:
+ * @bs: the bytestream to flush
+ * @len: the number of bytes to flush
+ *
+ * Flush len bytes from the bytestream. 
+ * This function can return FALSE when the number of
+ * bytes could not be flushed due to an event. In that case,
+ * you can get the number of available bytes before the event
+ * with gst_bytestream_get_status().
+ *
+ * Returns: TRUE if the number of bytes could be flushed.
+ */
 gboolean
 gst_bytestream_flush (GstByteStream *bs, guint32 len)
 {
-  bs_print ("flush: flushing %d bytes", len);
+  GST_DEBUG ("flush: flushing %d bytes", len);
 
   if (len == 0)
     return TRUE;
 
   /* make sure we have enough */
-  bs_print ("flush: there are %d bytes in the list", bs->listavail);
+  GST_DEBUG ("flush: there are %d bytes in the list", bs->listavail);
   if (len > bs->listavail) {
-    if (!gst_bytestream_fill_bytes (bs, len))
+    if (!gst_bytestream_fill_bytes (bs, len)) {
       return FALSE;
-    bs_print ("flush: there are now %d bytes in the list", bs->listavail);
+    }
+    GST_DEBUG ("flush: there are now %d bytes in the list", bs->listavail);
   }
 
   gst_bytestream_flush_fast (bs, len);
@@ -394,6 +447,16 @@ gst_bytestream_flush (GstByteStream *bs, guint32 len)
   return TRUE;
 }
 
+/**
+ * gst_bytestream_flush_fast:
+ * @bs: the bytestream to flush
+ * @len: the number of bytes to flush
+ *
+ * Flushes len bytes from the bytestream. This function
+ * is faster than gst_bytestream_flush() but only works
+ * when you have recently peeked no less than len bytes
+ * with gst_bytestream_peek() or gst_bytestream_peek_bytes().
+ */
 void
 gst_bytestream_flush_fast (GstByteStream *bs, guint32 len)
 {
@@ -416,13 +479,13 @@ gst_bytestream_flush_fast (GstByteStream *bs, guint32 len)
   while (len > 0) {
     headbuf = GST_BUFFER (bs->buflist->data);
 
-    bs_print ("flush: analyzing buffer that's %d bytes long, offset %llu", GST_BUFFER_SIZE (headbuf),
+    GST_DEBUG ("flush: analyzing buffer that's %d bytes long, offset %" G_GUINT64_FORMAT, GST_BUFFER_SIZE (headbuf),
 	      GST_BUFFER_OFFSET (headbuf));
 
     /* if there's enough to complete the flush */
     if (bs->headbufavail > len) {
       /* just trim it off */
-      bs_print ("flush: trimming %d bytes off end of headbuf", len);
+      GST_DEBUG ("flush: trimming %d bytes off end of headbuf", len);
       bs->headbufavail -= len;
       bs->listavail -= len;
       len = 0;
@@ -430,7 +493,7 @@ gst_bytestream_flush_fast (GstByteStream *bs, guint32 len)
       /* otherwise we have to trim the whole buffer */
     }
     else {
-      bs_print ("flush: removing head buffer completely");
+      GST_DEBUG ("flush: removing head buffer completely");
       /* remove it from the list */
       bs->buflist = g_slist_delete_link (bs->buflist, bs->buflist);
       /* trim it from the avail size */
@@ -443,17 +506,31 @@ gst_bytestream_flush_fast (GstByteStream *bs, guint32 len)
       /* record the new headbufavail */
       if (bs->buflist) {
 	bs->headbufavail = GST_BUFFER_SIZE (GST_BUFFER (bs->buflist->data));
-	bs_print ("flush: next headbuf is %d bytes", bs->headbufavail);
+	GST_DEBUG ("flush: next headbuf is %d bytes", bs->headbufavail);
       }
       else {
-	bs_print ("flush: no more bytes at all");
+	GST_DEBUG ("flush: no more bytes at all");
       }
     }
 
-    bs_print ("flush: bottom of while(), len is now %d", len);
+    GST_DEBUG ("flush: bottom of while(), len is now %d", len);
   }
 }
 
+/**
+ * gst_bytestream_seek:
+ * @bs: the bytestream to seek
+ * @offset: the byte offset to seek to
+ * @method: the seek method.
+ *
+ * Perform a seek on the bytestream to the given offset.
+ * The method can be one of GST_SEEK_METHOD_CUR, GST_SEEK_METHOD_SET,
+ * GST_SEEK_METHOD_END.
+ * This seek will also flush any pending data in the bytestream or
+ * peer elements.
+ *
+ * Returns: TRUE when the seek succeeded.
+ */
 gboolean
 gst_bytestream_seek (GstByteStream *bs, gint64 offset, GstSeekType method)
 {
@@ -463,7 +540,7 @@ gst_bytestream_seek (GstByteStream *bs, gint64 offset, GstSeekType method)
   
   peer = GST_RPAD_PEER (bs->pad);
 
-  bs_print ("bs: send event\n");
+  GST_DEBUG ("bs: send event\n");
   if (gst_pad_send_event (GST_PAD (peer), gst_event_new_seek (
 			  GST_FORMAT_BYTES | 
 			  (method & GST_SEEK_METHOD_MASK) | 
@@ -478,10 +555,18 @@ gst_bytestream_seek (GstByteStream *bs, gint64 offset, GstSeekType method)
     
     return TRUE;
   }
-  bs_print ("bs: send event failed\n");
+  GST_DEBUG ("bs: send event failed\n");
   return FALSE;
 }
 
+/**
+ * gst_bytestream_tell
+ * @bs: a bytestream
+ *
+ * Get the current byteoffset in the bytestream.
+ *
+ * Returns: the offset or -1 on error.
+ */
 guint64
 gst_bytestream_tell (GstByteStream *bs)
 {
@@ -499,6 +584,14 @@ gst_bytestream_tell (GstByteStream *bs)
   return -1;
 }
 
+/**
+ * gst_bytestream_length
+ * @bs: a bytestream
+ *
+ * Get the total length of the bytestream.
+ *
+ * Returns: the total length or -1 on error.
+ */
 guint64
 gst_bytestream_length (GstByteStream *bs)
 {
@@ -515,6 +608,20 @@ gst_bytestream_length (GstByteStream *bs)
   return -1;
 }
 
+/**
+ * gst_bytestream_read:
+ * @bs: the bytestream to read
+ * @buf: pointer to a variable that can hold a buffer pointer.
+ * @len: the number of bytes to read
+ *
+ * Read len bytes from the bytestream, the result is returned as
+ * a #GstBuffer. Unref the buffer after usage.
+ * This function can return less bytes than requested. In that case,
+ * an event might have happened which you can retrieve with
+ * gst_bytestream_get_status().
+ *
+ * Returns: The number of bytes successfully read.
+ */
 guint32
 gst_bytestream_read (GstByteStream *bs, GstBuffer** buf, guint32 len)
 {
@@ -536,7 +643,9 @@ gst_bytestream_read (GstByteStream *bs, GstBuffer** buf, guint32 len)
  * @bs: a bytestream
  * @size: the size to hint
  *
- * Give a hint that we are going to read chunks of the given size
+ * Give a hint that we are going to read chunks of the given size.
+ * Giving size hints to the peer element might improve performance
+ * since less buffers need to be merged.
  *
  * Returns: TRUE if the hint was accepted
  */
@@ -558,13 +667,9 @@ gst_bytestream_size_hint (GstByteStream *bs, guint32 size)
  * @avail_out: total number of bytes buffered
  * @event_out: an event
  *
- * When an event occurs, the bytestream will return NULL.  You must
- * retrieve the event using this API before reading more bytes from
- * the stream.
- *
- * It is possible for the bytestream to return NULL due to running
- * out of buffers, however, this indicates a bug because an EOS
- * event should have been sent.
+ * When an event occurs, the bytestream operations return a value less
+ * than the requested length. You must retrieve the event using this API 
+ * before reading more bytes from the stream.
  */
 void
 gst_bytestream_get_status (GstByteStream *bs,
@@ -590,6 +695,8 @@ gst_bytestream_get_status (GstByteStream *bs,
  * This function will not check input buffer boundries.  It is  possible
  * the next read could span two or more input buffers with different
  * timestamps.
+ *
+ * Returns: a timestamp
  */
 guint64
 gst_bytestream_get_timestamp (GstByteStream *bs)
@@ -598,11 +705,11 @@ gst_bytestream_get_timestamp (GstByteStream *bs)
 
   g_return_val_if_fail (bs != NULL, 0);
 
-  bs_print ("get_timestamp: getting timestamp");
+  GST_DEBUG ("get_timestamp: getting timestamp");
 
   /* make sure we have a buffer */
   if (bs->listavail == 0) {
-    bs_print ("gst_timestamp: fetching a buffer");
+    GST_DEBUG ("gst_timestamp: fetching a buffer");
     if (!gst_bytestream_fill_bytes (bs, 1))
       return 0;
   }
@@ -613,34 +720,48 @@ gst_bytestream_get_timestamp (GstByteStream *bs)
   return GST_BUFFER_TIMESTAMP (headbuf);
 }
 
+/**
+ * gst_bytestream_print_status
+ * @bs: a bytestream
+ *
+ * Print the current status of the bytestream object. mainly
+ * used for debugging purposes.
+ */
 void
 gst_bytestream_print_status (GstByteStream * bs)
 {
   GSList *walk;
   GstBuffer *buf;
 
-  bs_print ("STATUS: head buffer has %d bytes available", bs->headbufavail);
-  bs_print ("STATUS: list has %d bytes available", bs->listavail);
+  GST_DEBUG ("STATUS: head buffer has %d bytes available", bs->headbufavail);
+  GST_DEBUG ("STATUS: list has %d bytes available", bs->listavail);
   walk = bs->buflist;
   while (walk) {
     buf = GST_BUFFER (walk->data);
     walk = g_slist_next (walk);
 
-    bs_print ("STATUS: buffer starts at %llu and is %d bytes long", GST_BUFFER_OFFSET (buf), GST_BUFFER_SIZE (buf));
+    GST_DEBUG ("STATUS: buffer starts at %" G_GUINT64_FORMAT " and is %d bytes long", 
+	      GST_BUFFER_OFFSET (buf), GST_BUFFER_SIZE (buf));
   }
 }
 
-
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  gst_plugin_set_longname (plugin, "GstByteStream: a byte-oriented layer on top of buffer-passing");
+  GST_DEBUG_CATEGORY_INIT (debug_bs, "bytestream", 0, "bytestream library");
+
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "gstbytestream",
-  plugin_init
-};
+  "a byte-oriented layer on top of buffer-passing",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
