@@ -1,3 +1,4 @@
+#include <stdlib.h>
 #include <gst/gst.h>
 
 /* this pipeline is:
@@ -28,13 +29,12 @@ int main(int argc,char *argv[])
   for (x = 0 ; x < 10 ; x++){
     g_print("playing %d\n", x);
     gst_element_set_state(GST_ELEMENT(thread), GST_STATE_PLAYING);
-    sleep(1);
+    g_usleep (G_USEC_PER_SEC);
 
     g_print("pausing %d\n", x);
     gst_element_set_state(GST_ELEMENT(thread), GST_STATE_PAUSED);
-    sleep(1);
+    g_usleep (G_USEC_PER_SEC);
   }
 
   exit(0);
 }
-
