@@ -1,3 +1,4 @@
+#include <stdlib.h>
 #include <gst/gst.h>
 
 /* this pipeline is:
@@ -32,12 +33,11 @@ int main(int argc,char *argv[])
   for (x = 0 ; x < 10 ; x++){
     g_print("playing %d\n", x);
     gst_element_set_state(GST_ELEMENT(thread), GST_STATE_PLAYING);
-    sleep(1);
+    g_usleep (G_USEC_PER_SEC);
 
     g_print("nulling %d\n", x);
     gst_element_set_state(GST_ELEMENT(thread), GST_STATE_NULL);
-    sleep(1);
+    g_usleep (G_USEC_PER_SEC);
   }
   exit(0);
 }
-
