@@ -21,6 +21,10 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gst/gst.h>
 #include <gst/control/control.h>
@@ -55,22 +59,13 @@ struct _GstDpTestClass
 
 GType gst_dptest_get_type (void);
 
-
-GstElementDetails gst_dptest_details = {
-  "DParamsTest",
-  "Filter",
-  "Test for the GstDParam code",
-  VERSION,
-  "Steve Baker <stevebaker_org@yahoo.co.uk>",
-  "(C) 2001",
-};
-
 enum
 {
   ARG_0,
 };
 
 
+static void     gst_dptest_base_init            (gpointer g_class);
 static void 	gst_dptest_class_init 		(GstDpTestClass * klass);
 static void 	gst_dptest_init 		(GstDpTest * dptest);
 
@@ -78,7 +73,7 @@ static void gst_dptest_set_property (GObject * object, guint prop_id, const GVal
 				     GParamSpec * pspec);
 
 static GstElementStateReturn 	gst_dptest_change_state 	(GstElement *element);
-static void gst_dptest_chain (GstPad *pad, GstBuffer *buf);
+static void gst_dptest_chain (GstPad *pad, GstData *buf);
 
 static GstElementClass *parent_class = NULL;
 
@@ -89,7 +84,8 @@ gst_dptest_get_type (void)
 
   if (!dptest_type) {
     static const GTypeInfo dptest_info = {
-      sizeof (GstDpTestClass), NULL,
+      sizeof (GstDpTestClass),
+      gst_dptest_base_init,
       NULL,
       (GClassInitFunc) gst_dptest_class_init,
       NULL,
@@ -104,6 +100,22 @@ gst_dptest_get_type (void)
   return dptest_type;
 }
 
+static void
+gst_dptest_base_init (gpointer g_class)
+{
+  static GstElementDetails dptest_details = GST_ELEMENT_DETAILS (
+    "DParamTest",
+    "Filter",
+    "Test for the GstDParam code",
+    "Steve Baker <stevebaker_org@yahoo.co.uk>"
+  );
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (element_class, &dptest_details);
+
+  g_print("got here %d\n",__LINE__);
+}
+
 static void
 gst_dptest_class_init (GstDpTestClass * klass)
 {
@@ -178,10 +190,10 @@ gst_dptest_change_state (GstElement *element)
 }
 
 static void 
-gst_dptest_chain (GstPad *pad, GstBuffer *buf)
+gst_dptest_chain (GstPad *pad, GstData *data)
 {
   GstDpTest *dptest;
-  gint i=0, frame_countdown;
+  gint frame_countdown;
 
   dptest = GST_DPTEST(gst_pad_get_parent (pad));
   g_assert(dptest);
@@ -189,14 +201,34 @@ gst_dptest_chain (GstPad *pad, GstBuffer *buf)
   /* we're using a made up buffer size of 64 and a timestamp of zero */
   frame_countdown = GST_DPMAN_PREPROCESS(dptest->dpman, 64, 0LL);
   
-  while(GST_DPMAN_PROCESS_COUNTDOWN(dptest->dpman, frame_countdown, i));
+  while(GST_DPMAN_PROCESS(dptest->dpman, frame_countdown));
   	
   g_print("dp chain\n");
 }
 
+gboolean
+gst_dptest_register_elements (GstPlugin *plugin)
+{
+  return gst_element_register (plugin, "dptest", GST_RANK_NONE,
+      GST_TYPE_DPTEST);
+}
+
+static GstPluginDesc plugin_desc = {
+  GST_VERSION_MAJOR,
+  GST_VERSION_MINOR,
+  "dptest_elements",
+  "test elements",
+  &gst_dptest_register_elements,
+  NULL,
+  VERSION,
+  GST_LICENSE,
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+};
+
 int main(int argc,char *argv[]) {
 
-  GstElementFactory *factory;
   GstElement *src;
   GstElement *sink;
   GstElement *testelement;
@@ -205,11 +237,12 @@ int main(int argc,char *argv[]) {
   GstDParam *dp_float1;
   GValue *dp_float1_value;
   
+  alarm(10);
+
   gst_init (&argc, &argv);
   gst_control_init(&argc,&argv);
-  
-  factory = gst_element_factory_new ("dptest", GST_TYPE_DPTEST, &gst_dptest_details);
-  g_assert (factory != NULL);
+
+  _gst_plugin_register_static (&plugin_desc);
 
   pipeline = gst_element_factory_make ("pipeline", "pipeline");
   g_assert (pipeline);
