@@ -42,7 +42,7 @@ static gint num_plugins = 0;
 static void
 plugin_added_func (GstRegistry *registry, GstPlugin *plugin, gpointer user_data)
 {
-  g_print ("added plugin %s with %d feature(s)\n", plugin->name,
+  g_print ("added plugin %s with %d feature(s)\n", plugin->desc.name,
            plugin->numfeatures);
 
   num_features += plugin->numfeatures;
@@ -62,7 +62,7 @@ spawn_all_in_dir(const char *dirname)
   if (dir == NULL) return;
 
   while( (file = g_dir_read_name (dir)) ){
-    argv[0] = g_build_filename(dirname, file);
+    argv[0] = g_build_filename(dirname, file, NULL);
     g_print("running %s\n",argv[0]);
     g_spawn_sync (NULL, argv, NULL, G_SPAWN_FILE_AND_ARGV_ZERO, NULL, NULL, NULL, NULL, NULL, NULL);
     g_free(argv[0]);
@@ -132,7 +132,7 @@ int main (int argc,char *argv[])
 
     dir_list = gst_registry_get_path_list(registry);
     for(iter = dir_list; iter; iter = iter->next) {
-      dir = g_build_filename((const char *)iter->data, "register-scripts");
+      dir = g_build_filename((const char *)iter->data, "register-scripts", NULL);
       spawn_all_in_dir(dir);
       g_free(dir);
     }
